@@ -5,11 +5,10 @@ import java.math.BigDecimal;
 import java.math.BigInteger;
 import java.util.*;
 
-import com.fasterxml.jackson.core.JsonPointer;
-import com.fasterxml.jackson.core.TreeNode;
+import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.node.JsonNodeType;
 import com.fasterxml.jackson.databind.node.MissingNode;
-import com.fasterxml.jackson.databind.util.EmptyIterator;
+import com.fasterxml.jackson.databind.util.ClassUtil;
 
 /**
  * Base class for all JSON nodes, which form the basis of JSON
@@ -29,8 +28,16 @@ import com.fasterxml.jackson.databind.util.EmptyIterator;
  *<p>
  * Actual concrete sub-classes can be found from package
  * {@link com.fasterxml.jackson.databind.node}.
+ *<p>
+ * Note that it is possible to "read" from nodes, using
+ * method {@link TreeNode#traverse(ObjectCodec)}, which will result in
+ * a {@link JsonParser} being constructed. This can be used for (relatively)
+ * efficient conversations between different representations; and it is what
+ * core databind uses for methods like {@link ObjectMapper#treeToValue(TreeNode, Class)}
+ * and {@link ObjectMapper#treeAsTokens(TreeNode)}
  */
 public abstract class JsonNode
+    extends JsonSerializable.Base // i.e. implements JsonSerializable
     implements TreeNode, Iterable<JsonNode>
 {
     /*
@@ -66,11 +73,10 @@ public abstract class JsonNode
     /**********************************************************
      */
 
-//    public abstract JsonToken asToken();
-
-//    public abstract JsonParser.NumberType numberType();
-
-//    public abstract JsonParser traverse();
+//  public abstract JsonToken asToken();
+//  public abstract JsonToken traverse();
+//  public abstract JsonToken traverse(ObjectCodec codec);
+//  public abstract JsonParser.NumberType numberType();
 
     @Override
     public int size() { return 0; }
@@ -93,18 +99,18 @@ public abstract class JsonNode
     }
 
     @Override
-    public final boolean isMissingNode() {
-        return getNodeType() == JsonNodeType.MISSING;
+    public boolean isMissingNode() {
+        return false;
     }
 
     @Override
-    public final boolean isArray() {
-        return getNodeType() == JsonNodeType.ARRAY;
+    public boolean isArray() {
+        return false;
     }
 
     @Override
-    public final boolean isObject() {
-        return getNodeType() == JsonNodeType.OBJECT;
+    public boolean isObject() {
+        return false;
     }
 
     /**
@@ -174,7 +180,7 @@ public abstract class JsonNode
 
     @Override
     public Iterator<String> fieldNames() {
-        return EmptyIterator.instance();
+        return ClassUtil.emptyIterator();
     }
 
     /**
@@ -225,7 +231,7 @@ public abstract class JsonNode
     }
 
     protected abstract JsonNode _at(JsonPointer ptr);
-    
+
     /*
     /**********************************************************
     /* Public API, type introspection
@@ -502,9 +508,24 @@ public abstract class JsonNode
      */
     public double doubleValue() { return 0.0; }
 
+    /**
+     * Returns floating point value for this node (as {@link BigDecimal}), <b>if and only if</b>
+     * this node is numeric ({@link #isNumber} returns true). For other
+     * types returns <code>BigDecimal.ZERO</code>.
+     *
+     * @return {@link BigDecimal} value this node contains, if numeric node; <code>BigDecimal.ZERO</code> for non-number nodes.
+     */
     public BigDecimal decimalValue() { return BigDecimal.ZERO; }
+
+    /**
+     * Returns integer value for this node (as {@link BigDecimal}), <b>if and only if</b>
+     * this node is numeric ({@link #isNumber} returns true). For other
+     * types returns <code>BigInteger.ZERO</code>.
+     *
+     * @return {@link BigInteger} value this node contains, if numeric node; <code>BigInteger.ZERO</code> for non-number nodes.
+     */
     public BigInteger bigIntegerValue() { return BigInteger.ZERO; }
-    
+
     /*
     /**********************************************************
     /* Public API, value access with conversion(s)/coercion(s)
@@ -538,7 +559,7 @@ public abstract class JsonNode
      * and 1 (true), and Strings are parsed using default Java language integer
      * parsing rules.
      *<p>
-     * If representation can not be converted to an int (including structured types
+     * If representation cannot be converted to an int (including structured types
      * like Objects and Arrays),
      * default value of <b>0</b> will be returned; no exceptions are thrown.
      */
@@ -552,7 +573,7 @@ public abstract class JsonNode
      * and 1 (true), and Strings are parsed using default Java language integer
      * parsing rules.
      *<p>
-     * If representation can not be converted to an int (including structured types
+     * If representation cannot be converted to an int (including structured types
      * like Objects and Arrays),
      * specified <b>defaultValue</b> will be returned; no exceptions are thrown.
      */
@@ -566,7 +587,7 @@ public abstract class JsonNode
      * and 1 (true), and Strings are parsed using default Java language integer
      * parsing rules.
      *<p>
-     * If representation can not be converted to an long (including structured types
+     * If representation cannot be converted to an long (including structured types
      * like Objects and Arrays),
      * default value of <b>0</b> will be returned; no exceptions are thrown.
      */
@@ -580,7 +601,7 @@ public abstract class JsonNode
      * and 1 (true), and Strings are parsed using default Java language integer
      * parsing rules.
      *<p>
-     * If representation can not be converted to an long (including structured types
+     * If representation cannot be converted to an long (including structured types
      * like Objects and Arrays),
      * specified <b>defaultValue</b> will be returned; no exceptions are thrown.
      */
@@ -594,7 +615,7 @@ public abstract class JsonNode
      * and 1.0 (true), and Strings are parsed using default Java language integer
      * parsing rules.
      *<p>
-     * If representation can not be converted to an int (including structured types
+     * If representation cannot be converted to an int (including structured types
      * like Objects and Arrays),
      * default value of <b>0.0</b> will be returned; no exceptions are thrown.
      */
@@ -608,7 +629,7 @@ public abstract class JsonNode
      * and 1.0 (true), and Strings are parsed using default Java language integer
      * parsing rules.
      *<p>
-     * If representation can not be converted to an int (including structured types
+     * If representation cannot be converted to an int (including structured types
      * like Objects and Arrays),
      * specified <b>defaultValue</b> will be returned; no exceptions are thrown.
      */
@@ -622,7 +643,7 @@ public abstract class JsonNode
      * 0 maps to false
      * and Strings 'true' and 'false' map to corresponding values.
      *<p>
-     * If representation can not be converted to a boolean value (including structured types
+     * If representation cannot be converted to a boolean value (including structured types
      * like Objects and Arrays),
      * default value of <b>false</b> will be returned; no exceptions are thrown.
      */
@@ -636,20 +657,20 @@ public abstract class JsonNode
      * 0 maps to false
      * and Strings 'true' and 'false' map to corresponding values.
      *<p>
-     * If representation can not be converted to a boolean value (including structured types
+     * If representation cannot be converted to a boolean value (including structured types
      * like Objects and Arrays),
      * specified <b>defaultValue</b> will be returned; no exceptions are thrown.
      */
     public boolean asBoolean(boolean defaultValue) {
         return defaultValue;
     }
-    
+
     /*
     /**********************************************************
     /* Public API, value find / existence check methods
     /**********************************************************
      */
-    
+
     /**
      * Method that allows checking whether this node is JSON Object node
      * and contains value for specified property. If this is the case
@@ -706,7 +727,7 @@ public abstract class JsonNode
      *<p>
      * This method is functionally equivalent to:
      *<pre>
-     *   node.get(fieldName) != null && !node.get(fieldName).isNull()
+     *   node.get(fieldName) != null &lt;&lt; !node.get(fieldName).isNull()
      *</pre>
      * 
      * @since 2.1
@@ -722,7 +743,7 @@ public abstract class JsonNode
      *<p>
      * This method is equivalent to:
      *<pre>
-     *   node.get(index) != null && !node.get(index).isNull()
+     *   node.get(index) != null &lt;&lt; !node.get(index).isNull()
      *</pre>
      * 
      * @since 2.1
@@ -753,7 +774,7 @@ public abstract class JsonNode
      * For other types of nodes, returns empty iterator.
      */
     public Iterator<JsonNode> elements() {
-        return EmptyIterator.instance();
+        return ClassUtil.emptyIterator();
     }
 
     /**
@@ -761,15 +782,15 @@ public abstract class JsonNode
      *   object nodes; empty iterator (no contents) for other types
      */
     public Iterator<Map.Entry<String, JsonNode>> fields() {
-        return EmptyIterator.instance();
+        return ClassUtil.emptyIterator();
     }
-    
+
     /*
     /**********************************************************
     /* Public API, find methods
     /**********************************************************
      */
-    
+
     /**
      * Method for finding a JSON Object field with specified name in this
      * node or its child nodes, and returning value it has.
@@ -875,7 +896,7 @@ public abstract class JsonNode
      */
     public JsonNode with(String propertyName) {
         throw new UnsupportedOperationException("JsonNode not of type ObjectNode (but "
-                +getClass().getName()+"), can not call with() on it");
+                +getClass().getName()+"), cannot call with() on it");
     }
 
     /**
@@ -888,9 +909,37 @@ public abstract class JsonNode
      */
     public JsonNode withArray(String propertyName) {
         throw new UnsupportedOperationException("JsonNode not of type ObjectNode (but "
-                +getClass().getName()+"), can not call withArray() on it");
+                +getClass().getName()+"), cannot call withArray() on it");
     }
 
+    /*
+    /**********************************************************
+    /* Public API, comparison
+    /**********************************************************
+     */
+
+    /**
+     * Entry method for invoking customizable comparison, using passed-in
+     * {@link Comparator} object. Nodes will handle traversal of structured
+     * types (arrays, objects), but defer to comparator for scalar value
+     * comparisons. If a "natural" {@link Comparator} is passed -- one that
+     * simply calls <code>equals()</code> on one of arguments, passing the other
+     * -- implementation is the same as directly calling <code>equals()</code>
+     * on node.
+     *<p>
+     * Default implementation simply delegates to passed in <code>comparator</code>,
+     * with <code>this</code> as the first argument, and <code>other</code> as
+     * the second argument.
+     * 
+     * @param comparator Object called to compare two scalar {@link JsonNode} 
+     *   instances, and return either 0 (are equals) or non-zero (not equal)
+     *
+     * @since 2.6
+     */
+    public boolean equals(Comparator<JsonNode> comparator, JsonNode other) {
+        return comparator.compare(this, other) == 0;
+    }
+    
     /*
     /**********************************************************
     /* Overridden standard methods
@@ -898,9 +947,19 @@ public abstract class JsonNode
      */
     
     /**
+     * Method that will produce developer-readable representation of the
+     * node; which may <b>or may not</b> be as valid JSON.
+     * If you want valid JSON output (or output formatted using one of
+     * other Jackson supported data formats) make sure to use
+     * {@link ObjectMapper} or {@link ObjectWriter} to serialize an
+     * instance, for example:
+     *<pre>
+     *   String json = objectMapper.writeValueAsString(rootNode);
+     *</pre>
      *<p>
-     * Note: marked as abstract to ensure all implementation
-     * classes define it properly.
+     * Note: method defined as abstract to ensure all implementation
+     * classes explicitly implement method, instead of relying
+     * on {@link Object#toString()} definition.
      */
     @Override
     public abstract String toString();
