@@ -4,15 +4,15 @@ import java.io.IOException;
 import java.lang.annotation.Annotation;
 
 import com.fasterxml.jackson.core.*;
+
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.deser.impl.FailingDeserializer;
-import com.fasterxml.jackson.databind.deser.impl.NullProvider;
-import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
-import com.fasterxml.jackson.databind.introspect.BeanPropertyDefinition;
-import com.fasterxml.jackson.databind.introspect.ObjectIdInfo;
+import com.fasterxml.jackson.databind.deser.impl.NullsConstantProvider;
+import com.fasterxml.jackson.databind.introspect.*;
 import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonObjectFormatVisitor;
 import com.fasterxml.jackson.databind.jsontype.TypeDeserializer;
 import com.fasterxml.jackson.databind.util.Annotations;
+import com.fasterxml.jackson.databind.util.ClassUtil;
 import com.fasterxml.jackson.databind.util.ViewMatcher;
 
 /**
@@ -24,8 +24,8 @@ import com.fasterxml.jackson.databind.util.ViewMatcher;
  */
 @SuppressWarnings("serial")
 public abstract class SettableBeanProperty
-    implements BeanProperty,
-        java.io.Serializable
+    extends ConcreteBeanPropertyBase
+    implements java.io.Serializable
 {
     /**
      * To avoid nasty NPEs, let's use a placeholder for _valueDeserializer,
@@ -35,7 +35,7 @@ public abstract class SettableBeanProperty
      */
     protected static final JsonDeserializer<Object> MISSING_VALUE_DESERIALIZER = new FailingDeserializer(
             "No _valueDeserializer assigned");
-    
+
     /**
      * Logical name of the property (often but not always derived
      * from the setter method name)
@@ -51,14 +51,14 @@ public abstract class SettableBeanProperty
      * @since 2.2
      */
     protected final PropertyName _wrapperName;
-    
+
     /**
      * Class that contains this property (either class that declares
      * the property or one of its subclasses), class that is
      * deserialized using deserializer that contains this property.
      */
     protected final transient Annotations _contextAnnotations;
-    
+
     /**
      * Deserializer used for handling property value.
      *<p>
@@ -72,35 +72,27 @@ public abstract class SettableBeanProperty
      * used to handle type resolution.
      */
     protected final TypeDeserializer _valueTypeDeserializer;
-    
-    /**
-     * Object used to figure out value to be used when 'null' literal is encountered in JSON.
-     * For most types simply Java null, but for primitive types must
-     * be a non-null value (like Integer.valueOf(0) for int).
-     */
-    protected final NullProvider _nullProvider;
 
     /**
-     * Additional optional property metadata, such as whether
-     * property is required, and whether there is additional
-     * human-readable description
-     * 
-     * @since 2.3
+     * Entity used for possible translation from `null` into non-null
+     * value of type of this property.
+     * Often same as <code>_valueDeserializer</code>, but not always.
+     *
+     * @since 2.9
      */
-    protected final PropertyMetadata _metadata;
+    protected final NullValueProvider _nullProvider;
 
     /*
     /**********************************************************
     /* Configuration that is not yet immutable; generally assigned
-    /* during initialization process but can not be passed to
+    /* during initialization process but cannot be passed to
     /* constructor.
     /**********************************************************
      */
 
     /**
-     * If property represents a managed (forward) reference
-     * (see [JACKSON-235]), we will need name of reference for
-     * later linking.
+     * If property represents a managed (forward) reference, we will need
+     * the name of reference for later linking.
      *<p>
      * TODO: should try to make immutable.
      */
@@ -121,7 +113,7 @@ public abstract class SettableBeanProperty
      * TODO: should try to make immutable.
      */
     protected ViewMatcher _viewMatcher;
-    
+
     /**
      * Index of property (within all property of a bean); assigned
      * when all properties have been collected. Order of entries
@@ -145,27 +137,11 @@ public abstract class SettableBeanProperty
                 contextAnnotations, propDef.getMetadata());
     }
 
-    @Deprecated // since 2.2
-    protected SettableBeanProperty(String propName, JavaType type, PropertyName wrapper,
-            TypeDeserializer typeDeser, Annotations contextAnnotations)
-    {
-        this(new PropertyName(propName), type, wrapper, typeDeser, contextAnnotations,
-                PropertyMetadata.STD_OPTIONAL);
-    }
-
-    @Deprecated // since 2.3
-    protected SettableBeanProperty(String propName, JavaType type, PropertyName wrapper,
-            TypeDeserializer typeDeser, Annotations contextAnnotations,
-            boolean isRequired)
-    {
-        this(new PropertyName(propName), type, wrapper, typeDeser, contextAnnotations,
-                PropertyMetadata.construct(isRequired, null, null));
-    }
-    
     protected SettableBeanProperty(PropertyName propName, JavaType type, PropertyName wrapper,
             TypeDeserializer typeDeser, Annotations contextAnnotations,
             PropertyMetadata metadata)
     {
+        super(metadata);
         // 09-Jan-2009, tatu: Intern()ing makes sense since Jackson parsed
         //  field names are (usually) interned too, hence lookups will be faster.
         // 23-Oct-2009, tatu: should this be disabled wrt [JACKSON-180]?
@@ -178,10 +154,8 @@ public abstract class SettableBeanProperty
         }
         _type = type;
         _wrapperName = wrapper;
-        _metadata = metadata;
         _contextAnnotations = contextAnnotations;
         _viewMatcher = null;
-        _nullProvider = null;
 
         // 30-Jan-2012, tatu: Important: contextualize TypeDeserializer now...
         if (typeDeser != null) {
@@ -189,6 +163,7 @@ public abstract class SettableBeanProperty
         }
         _valueTypeDeserializer = typeDeser;
         _valueDeserializer = MISSING_VALUE_DESERIALIZER;
+        _nullProvider = MISSING_VALUE_DESERIALIZER;
     }
 
     /**
@@ -199,6 +174,7 @@ public abstract class SettableBeanProperty
     protected SettableBeanProperty(PropertyName propName, JavaType type, 
             PropertyMetadata metadata, JsonDeserializer<Object> valueDeser)
     {
+        super(metadata);
         // as with above ctor, intern()ing probably fine
         if (propName == null) {
             _propName = PropertyName.NO_NAME;
@@ -207,79 +183,77 @@ public abstract class SettableBeanProperty
         }
         _type = type;
         _wrapperName = null;
-        _metadata = metadata;
         _contextAnnotations = null;
         _viewMatcher = null;
-        _nullProvider = null;
         _valueTypeDeserializer = null;
         _valueDeserializer = valueDeser;
+        // 29-Jan-2017, tatu: Presumed to be irrelevant for ObjectId values...
+        _nullProvider = valueDeser;
     }
-    
+
     /**
      * Basic copy-constructor for sub-classes to use.
      */
     protected SettableBeanProperty(SettableBeanProperty src)
     {
+        super(src);
         _propName = src._propName;
         _type = src._type;
         _wrapperName = src._wrapperName;
-        _metadata = src._metadata;
         _contextAnnotations = src._contextAnnotations;
         _valueDeserializer = src._valueDeserializer;
         _valueTypeDeserializer = src._valueTypeDeserializer;
-        _nullProvider = src._nullProvider;
         _managedReferenceName = src._managedReferenceName;
         _propertyIndex = src._propertyIndex;
         _viewMatcher = src._viewMatcher;
+        _nullProvider = src._nullProvider;
     }
 
     /**
      * Copy-with-deserializer-change constructor for sub-classes to use.
      */
     @SuppressWarnings("unchecked")
-    protected SettableBeanProperty(SettableBeanProperty src, JsonDeserializer<?> deser)
+    protected SettableBeanProperty(SettableBeanProperty src,
+            JsonDeserializer<?> deser, NullValueProvider nuller)
     {
+        super(src);
         _propName = src._propName;
         _type = src._type;
         _wrapperName = src._wrapperName;
-        _metadata = src._metadata;
         _contextAnnotations = src._contextAnnotations;
         _valueTypeDeserializer = src._valueTypeDeserializer;
         _managedReferenceName = src._managedReferenceName;
         _propertyIndex = src._propertyIndex;
 
         if (deser == null) {
-            _nullProvider = null;
             _valueDeserializer = MISSING_VALUE_DESERIALIZER;
         } else {
-            Object nvl = deser.getNullValue();
-            _nullProvider = (nvl == null) ? null : new NullProvider(_type, nvl);
             _valueDeserializer = (JsonDeserializer<Object>) deser;
         }
         _viewMatcher = src._viewMatcher;
+        // 29-Jan-2017, tatu: Bit messy, but for now has to do...
+        if (nuller == MISSING_VALUE_DESERIALIZER) {
+            nuller = _valueDeserializer;
+        }
+        _nullProvider = nuller;
     }
 
-    @Deprecated // since 2.3
-    protected SettableBeanProperty(SettableBeanProperty src, String newName) {
-        this(src, new PropertyName(newName));
-    }
-    
     /**
      * Copy-with-deserializer-change constructor for sub-classes to use.
      */
     protected SettableBeanProperty(SettableBeanProperty src, PropertyName newName)
     {
+        super(src);
         _propName = newName;
         _type = src._type;
         _wrapperName = src._wrapperName;
-        _metadata = src._metadata;
         _contextAnnotations = src._contextAnnotations;
         _valueDeserializer = src._valueDeserializer;
         _valueTypeDeserializer = src._valueTypeDeserializer;
-        _nullProvider = src._nullProvider;
         _managedReferenceName = src._managedReferenceName;
         _propertyIndex = src._propertyIndex;
         _viewMatcher = src._viewMatcher;
+        _nullProvider = src._nullProvider;
     }
 
     /**
@@ -314,11 +288,11 @@ public abstract class SettableBeanProperty
                 ? new PropertyName(simpleName) : _propName.withSimpleName(simpleName);
         return (n == _propName) ? this : withName(n);
     }
-    
-    @Deprecated // since 2.3 -- use 'withSimpleName' instead if need be
-    public SettableBeanProperty withName(String simpleName) {
-        return withName(new PropertyName(simpleName));
-    }
+
+    /**
+     * @since 2.9
+     */
+    public abstract SettableBeanProperty withNullProvider(NullValueProvider nva);
 
     public void setManagedReferenceName(String n) {
         _managedReferenceName = n;
@@ -335,7 +309,7 @@ public abstract class SettableBeanProperty
             _viewMatcher = ViewMatcher.construct(views);
         }
     }
-    
+
     /**
      * Method used to assign index for property.
      */
@@ -345,7 +319,28 @@ public abstract class SettableBeanProperty
         }
         _propertyIndex = index;
     }
-    
+
+    /**
+     * Method called to ensure that the mutator has proper access rights to
+     * be called, as per configuration. Overridden by implementations that
+     * have mutators that require access, fields and setters.
+     *
+     * @since 2.8.3
+     */
+    public void fixAccess(DeserializationConfig config) {
+        ;
+    }
+
+    /**
+     * @since 2.9.4
+     */
+    public void markAsIgnorable() { }
+
+    /**
+     * @since 2.9.4
+     */
+    public boolean isIgnorable() { return false; }
+
     /*
     /**********************************************************
     /* BeanProperty impl
@@ -361,13 +356,7 @@ public abstract class SettableBeanProperty
     public PropertyName getFullName() {
         return _propName;
     }
-    
-    @Override
-    public boolean isRequired() { return _metadata.isRequired(); }
 
-    @Override
-    public PropertyMetadata getMetadata() { return _metadata; }
-    
     @Override
     public JavaType getType() { return _type; }
 
@@ -377,10 +366,10 @@ public abstract class SettableBeanProperty
     }
     
     @Override
-    public abstract <A extends Annotation> A getAnnotation(Class<A> acls);
+    public abstract AnnotatedMember getMember();
 
     @Override
-    public abstract AnnotatedMember getMember();
+    public abstract <A extends Annotation> A getAnnotation(Class<A> acls);
 
     @Override
     public <A extends Annotation> A getContextAnnotation(Class<A> acls) {
@@ -388,7 +377,8 @@ public abstract class SettableBeanProperty
     }
 
     @Override
-    public void depositSchemaProperty(JsonObjectFormatVisitor objectVisitor)
+    public void depositSchemaProperty(JsonObjectFormatVisitor objectVisitor,
+            SerializerProvider provider)
         throws JsonMappingException
     {
         if (isRequired()) {
@@ -397,14 +387,14 @@ public abstract class SettableBeanProperty
             objectVisitor.optionalProperty(this);
         }
     }
-    
+
     /*
     /**********************************************************
     /* Accessors
     /**********************************************************
      */
 
-    protected final Class<?> getDeclaringClass() {
+    protected Class<?> getDeclaringClass() {
         return getMember().getDeclaringClass();
     }
 
@@ -417,7 +407,7 @@ public abstract class SettableBeanProperty
     }
 
     public boolean hasValueTypeDeserializer() { return (_valueTypeDeserializer != null); }
-    
+
     public JsonDeserializer<Object> getValueDeserializer() {
         JsonDeserializer<Object> deser = _valueDeserializer;
         if (deser == MISSING_VALUE_DESERIALIZER) {
@@ -428,10 +418,15 @@ public abstract class SettableBeanProperty
 
     public TypeDeserializer getValueTypeDeserializer() { return _valueTypeDeserializer; }
 
+    /**
+     * @since 2.9
+     */
+    public NullValueProvider getNullValueProvider() { return _nullProvider; }
+
     public boolean visibleInView(Class<?> activeView) {
         return (_viewMatcher == null) || _viewMatcher.isVisibleForView(activeView);
     }
-    
+
     public boolean hasViews() { return _viewMatcher != null; }
     
     /**
@@ -449,8 +444,13 @@ public abstract class SettableBeanProperty
      * 
      * @since 2.1
      */
-    public int getCreatorIndex() { return -1; }
-    
+    public int getCreatorIndex() {
+        // changed from 'return -1' in 2.7.9 / 2.8.7
+        throw new IllegalStateException(String.format(
+                "Internal error: no creator index for property '%s' (of type %s)",
+                this.getName(), getClass().getName()));
+    }
+
     /**
      * Accessor for id of injectable value, if this bean property supports
      * value injection.
@@ -470,9 +470,8 @@ public abstract class SettableBeanProperty
      * that should be consumed to produce the value (the only value for
      * scalars, multiple for Objects and Arrays).
      */
-    public abstract void deserializeAndSet(JsonParser jp,
-    		DeserializationContext ctxt, Object instance)
-        throws IOException, JsonProcessingException;
+    public abstract void deserializeAndSet(JsonParser p,
+    		DeserializationContext ctxt, Object instance) throws IOException;
 
 	/**
 	 * Alternative to {@link #deserializeAndSet} that returns
@@ -482,9 +481,8 @@ public abstract class SettableBeanProperty
 	 *
 	 * @since 2.0
 	 */
-    public abstract Object deserializeSetAndReturn(JsonParser jp,
-    		DeserializationContext ctxt, Object instance)
-        throws IOException, JsonProcessingException;
+    public abstract Object deserializeSetAndReturn(JsonParser p,
+    		DeserializationContext ctxt, Object instance) throws IOException;
 
     /**
      * Method called to assign given value to this property, on
@@ -494,8 +492,7 @@ public abstract class SettableBeanProperty
      * implementations, creator-backed properties for example do not
      * support this method.
      */
-    public abstract void set(Object instance, Object value)
-        throws IOException;
+    public abstract void set(Object instance, Object value) throws IOException;
 
     /**
      * Method called to assign given value to this property, on
@@ -505,11 +502,8 @@ public abstract class SettableBeanProperty
      * Note: this is an optional operation, not supported by all
      * implementations, creator-backed properties for example do not
      * support this method.
-     * 
-     * @since 2.0
      */
-    public abstract Object setAndReturn(Object instance, Object value)
-            throws IOException;
+    public abstract Object setAndReturn(Object instance, Object value) throws IOException;
     
     /**
      * This method is needed by some specialized bean deserializers,
@@ -524,20 +518,42 @@ public abstract class SettableBeanProperty
      * this method should also not be called directly unless you really know
      * what you are doing (and probably not even then).
      */
-    public final Object deserialize(JsonParser jp, DeserializationContext ctxt)
-        throws IOException, JsonProcessingException
+    public final Object deserialize(JsonParser p, DeserializationContext ctxt) throws IOException
     {
-        JsonToken t = jp.getCurrentToken();
-        
-        if (t == JsonToken.VALUE_NULL) {
-            return (_nullProvider == null) ? null : _nullProvider.nullValue(ctxt);
+        if (p.hasToken(JsonToken.VALUE_NULL)) {
+            return _nullProvider.getNullValue(ctxt);
         }
         if (_valueTypeDeserializer != null) {
-            return _valueDeserializer.deserializeWithType(jp, ctxt, _valueTypeDeserializer);
+            return _valueDeserializer.deserializeWithType(p, ctxt, _valueTypeDeserializer);
         }
-        return _valueDeserializer.deserialize(jp, ctxt);
+        return _valueDeserializer.deserialize(p, ctxt);
     }
-    
+
+    /**
+     * @since 2.9
+     */
+    public final Object deserializeWith(JsonParser p, DeserializationContext ctxt,
+            Object toUpdate) throws IOException
+    {
+        // 20-Oct-2016, tatu: Not 100% sure what to do; probably best to simply return
+        //   null value and let caller decide what to do.
+        if (p.hasToken(JsonToken.VALUE_NULL)) {
+            // ... except for "skip nulls" case which should just do that:
+            if (NullsConstantProvider.isSkipper(_nullProvider)) {
+                return toUpdate;
+            }
+            return _nullProvider.getNullValue(ctxt);
+        }
+        // 20-Oct-2016, tatu: Also tricky -- for now, report an error
+        if (_valueTypeDeserializer != null) {
+            ctxt.reportBadDefinition(getType(),
+                    String.format("Cannot merge polymorphic property '%s'",
+                            getName()));
+//            return _valueDeserializer.deserializeWithType(p, ctxt, _valueTypeDeserializer);
+        }
+        return _valueDeserializer.deserialize(p, ctxt, toUpdate);
+    }
+
     /*
     /**********************************************************
     /* Helper methods
@@ -548,41 +564,206 @@ public abstract class SettableBeanProperty
      * Method that takes in exception of any type, and casts or wraps it
      * to an IOException or its subclass.
      */
-    protected void _throwAsIOE(Exception e, Object value)
-        throws IOException
+    protected void _throwAsIOE(JsonParser p, Exception e, Object value) throws IOException
     {
         if (e instanceof IllegalArgumentException) {
-            String actType = (value == null) ? "[NULL]" : value.getClass().getName();
-            StringBuilder msg = new StringBuilder("Problem deserializing property '").append(getName());
-            msg.append("' (expected type: ").append(getType());
-            msg.append("; actual type: ").append(actType).append(")");
+            String actType = ClassUtil.classNameOf(value);
+            StringBuilder msg = new StringBuilder("Problem deserializing property '")
+                    .append(getName())
+                    .append("' (expected type: ")
+                    .append(getType())
+                    .append("; actual type: ")
+                    .append(actType).append(")");
             String origMsg = e.getMessage();
             if (origMsg != null) {
-                msg.append(", problem: ").append(origMsg);
+                msg.append(", problem: ")
+                    .append(origMsg);
             } else {
                 msg.append(" (no error message provided)");
             }
-            throw new JsonMappingException(msg.toString(), null, e);
+            throw JsonMappingException.from(p, msg.toString(), e);
         }
-        _throwAsIOE(e);
+        _throwAsIOE(p, e);
+    }
+    
+    /**
+     * @since 2.7
+     */
+    protected IOException _throwAsIOE(JsonParser p, Exception e) throws IOException
+    {
+        ClassUtil.throwIfIOE(e);
+        ClassUtil.throwIfRTE(e);
+        // let's wrap the innermost problem
+        Throwable th = ClassUtil.getRootCause(e);
+        throw JsonMappingException.from(p, th.getMessage(), th);
     }
 
-    protected IOException _throwAsIOE(Exception e)
-        throws IOException
-    {
-        if (e instanceof IOException) {
-            throw (IOException) e;
-        }
-        if (e instanceof RuntimeException) {
-            throw (RuntimeException) e;
-        }
-        // let's wrap the innermost problem
-        Throwable th = e;
-        while (th.getCause() != null) {
-            th = th.getCause();
-        }
-        throw new JsonMappingException(th.getMessage(), null, th);
+    @Deprecated // since 2.7
+    protected IOException _throwAsIOE(Exception e) throws IOException {
+        return _throwAsIOE((JsonParser) null, e);
+    }
+
+    // 10-Oct-2015, tatu: _Should_ be deprecated, too, but its remaining
+    //   callers cannot actually provide a JsonParser
+    protected void _throwAsIOE(Exception e, Object value) throws IOException {
+        _throwAsIOE((JsonParser) null, e, value);
     }
 
     @Override public String toString() { return "[property '"+getName()+"']"; }
+
+    /*
+    /**********************************************************
+    /* Helper classes
+    /**********************************************************
+     */
+
+    /**
+     * Helper class that is designed to both make it easier to sub-class
+     * delegating subtypes and to reduce likelihood of breakage when
+     * new methods are added.
+     *<p>
+     * Class was specifically added to help with {@code Afterburner}
+     * module, but its use is not limited to only support it.
+     *
+     * @since 2.9
+     */
+    public static abstract class Delegating
+        extends SettableBeanProperty
+    {
+        protected final SettableBeanProperty delegate;
+
+        protected Delegating(SettableBeanProperty d) {
+            super(d);
+            delegate = d;
+        }
+
+        /**
+         * Method sub-classes must implement, to construct a new instance
+         * with given delegate.
+         */
+        protected abstract SettableBeanProperty withDelegate(SettableBeanProperty d);
+
+        protected SettableBeanProperty _with(SettableBeanProperty newDelegate) {
+            if (newDelegate == delegate) {
+                return this;
+            }
+            return withDelegate(newDelegate);
+        }
+        
+        @Override
+        public SettableBeanProperty withValueDeserializer(JsonDeserializer<?> deser) {
+            return _with(delegate.withValueDeserializer(deser));
+        }
+
+        @Override
+        public SettableBeanProperty withName(PropertyName newName) {
+            return _with(delegate.withName(newName));
+        }
+
+        @Override
+        public SettableBeanProperty withNullProvider(NullValueProvider nva) {
+            return _with(delegate.withNullProvider(nva));
+        }
+
+        @Override
+        public void assignIndex(int index) {
+            delegate.assignIndex(index);
+        }
+
+        @Override
+        public void fixAccess(DeserializationConfig config) {
+            delegate.fixAccess(config);
+        }
+
+        /*
+        /**********************************************************
+        /* Accessors
+        /**********************************************************
+         */
+
+        @Override
+        protected Class<?> getDeclaringClass() { return delegate.getDeclaringClass(); }
+
+        @Override
+        public String getManagedReferenceName() { return delegate.getManagedReferenceName(); }
+
+        @Override
+        public ObjectIdInfo getObjectIdInfo() { return delegate.getObjectIdInfo(); }
+
+        @Override
+        public boolean hasValueDeserializer() { return delegate.hasValueDeserializer(); }
+
+        @Override
+        public boolean hasValueTypeDeserializer() { return delegate.hasValueTypeDeserializer(); }
+        
+        @Override
+        public JsonDeserializer<Object> getValueDeserializer() { return delegate.getValueDeserializer(); }
+
+        @Override
+        public TypeDeserializer getValueTypeDeserializer() { return delegate.getValueTypeDeserializer(); }
+
+        @Override
+        public boolean visibleInView(Class<?> activeView) { return delegate.visibleInView(activeView); }
+
+        @Override
+        public boolean hasViews() { return delegate.hasViews(); }
+
+        @Override
+        public int getPropertyIndex() { return delegate.getPropertyIndex(); }
+
+        @Override
+        public int getCreatorIndex() { return delegate.getCreatorIndex(); }
+
+        @Override
+        public Object getInjectableValueId() { return delegate.getInjectableValueId(); }
+
+        @Override
+        public AnnotatedMember getMember() {
+            return delegate.getMember();
+        }
+
+        @Override
+        public <A extends Annotation> A getAnnotation(Class<A> acls) {
+            return delegate.getAnnotation(acls);
+        }
+
+        /*
+        /**********************************************************
+        /* Extended API
+        /**********************************************************
+         */
+
+        public SettableBeanProperty getDelegate() {
+            return delegate;
+        }
+
+        /*
+        /**********************************************************
+        /* Actual mutators
+        /**********************************************************
+         */
+
+        @Override
+        public void deserializeAndSet(JsonParser p, DeserializationContext ctxt,
+                Object instance) throws IOException {
+            delegate.deserializeAndSet(p, ctxt, instance);
+        }
+
+        @Override
+        public Object deserializeSetAndReturn(JsonParser p,
+                DeserializationContext ctxt, Object instance) throws IOException
+        {
+            return delegate.deserializeSetAndReturn(p, ctxt, instance);
+        }
+
+        @Override
+        public void set(Object instance, Object value) throws IOException {
+            delegate.set(instance, value);
+        }
+
+        @Override
+        public Object setAndReturn(Object instance, Object value) throws IOException {
+            return delegate.setAndReturn(instance, value);
+        }
+    }
 }
