@@ -4,6 +4,7 @@ import java.io.IOException;
 import java.lang.annotation.Annotation;
 
 import com.fasterxml.jackson.core.JsonParser;
+import com.fasterxml.jackson.core.JsonToken;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.deser.*;
 import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
@@ -20,11 +21,6 @@ public final class ObjectIdValueProperty
 
     protected final ObjectIdReader _objectIdReader;
 
-    @Deprecated // since 2.2
-    public ObjectIdValueProperty(ObjectIdReader objectIdReader) {
-        this(objectIdReader, PropertyMetadata.STD_REQUIRED);
-    }
-    
     public ObjectIdValueProperty(ObjectIdReader objectIdReader,
             PropertyMetadata metadata)
     {
@@ -33,33 +29,36 @@ public final class ObjectIdValueProperty
         _objectIdReader = objectIdReader;
     }
 
-    protected ObjectIdValueProperty(ObjectIdValueProperty src, JsonDeserializer<?> deser)
+    protected ObjectIdValueProperty(ObjectIdValueProperty src, JsonDeserializer<?> deser,
+            NullValueProvider nva)
     {
-        super(src, deser);
+        super(src, deser, nva);
         _objectIdReader = src._objectIdReader;
     }
 
-    @Deprecated // since 2.3
     protected ObjectIdValueProperty(ObjectIdValueProperty src, PropertyName newName) {
         super(src, newName);
         _objectIdReader = src._objectIdReader;
     }
-    
-    @Deprecated // since 2.3
-    protected ObjectIdValueProperty(ObjectIdValueProperty src, String newName) {
-        this(src, new PropertyName(newName));
-    }
 
     @Override
-    public ObjectIdValueProperty withName(PropertyName newName) {
+    public SettableBeanProperty withName(PropertyName newName) {
         return new ObjectIdValueProperty(this, newName);
     }
 
     @Override
-    public ObjectIdValueProperty withValueDeserializer(JsonDeserializer<?> deser) {
-        return new ObjectIdValueProperty(this, deser);
+    public SettableBeanProperty withValueDeserializer(JsonDeserializer<?> deser) {
+        if (_valueDeserializer == deser) {
+            return this;
+        }
+        return new ObjectIdValueProperty(this, deser, _nullProvider);
     }
-    
+
+    @Override
+    public SettableBeanProperty withNullProvider(NullValueProvider nva) {
+        return new ObjectIdValueProperty(this, _valueDeserializer, nva);
+    }
+
     // // // BeanProperty impl
     
     @Override
@@ -76,18 +75,26 @@ public final class ObjectIdValueProperty
      */
 
     @Override
-    public void deserializeAndSet(JsonParser jp, DeserializationContext ctxt,
+    public void deserializeAndSet(JsonParser p, DeserializationContext ctxt,
             Object instance) throws IOException
     {
-        deserializeSetAndReturn(jp, ctxt, instance);
+        deserializeSetAndReturn(p, ctxt, instance);
     }
 
     @Override
-    public Object deserializeSetAndReturn(JsonParser jp,
+    public Object deserializeSetAndReturn(JsonParser p,
     		DeserializationContext ctxt, Object instance) throws IOException
     {
+        /* 02-Apr-2015, tatu: Actually, as per [databind#742], let it be;
+         *  missing or null id is needed for some cases, such as cases where id
+         *  will be generated externally, at a later point, and is not available
+         *  quite yet. Typical use case is with DB inserts.
+         */
         // note: no null checks (unlike usually); deserializer should fail if one found
-        Object id = _valueDeserializer.deserialize(jp, ctxt);
+        if (p.hasToken(JsonToken.VALUE_NULL)) {
+            return null;
+        }
+        Object id = _valueDeserializer.deserialize(p, ctxt);
         ReadableObjectId roid = ctxt.findObjectId(id, _objectIdReader.generator, _objectIdReader.resolver);
         roid.bindItem(instance);
         // also: may need to set a property value as well
