@@ -5,6 +5,7 @@ import com.fasterxml.jackson.annotation.ObjectIdGenerator;
 import com.fasterxml.jackson.annotation.ObjectIdResolver;
 import com.fasterxml.jackson.annotation.SimpleObjectIdResolver;
 import com.fasterxml.jackson.databind.PropertyName;
+import com.fasterxml.jackson.databind.util.ClassUtil;
 
 /**
  * Container object that encapsulates information usually
@@ -15,27 +16,21 @@ public class ObjectIdInfo
 {
     protected final PropertyName _propertyName;
     protected final Class<? extends ObjectIdGenerator<?>> _generator;
-    private final Class<? extends ObjectIdResolver> _resolver;
+    protected final Class<? extends ObjectIdResolver> _resolver;
     protected final Class<?> _scope;
     protected final boolean _alwaysAsId;
 
+    /**
+     * @since 2.8.9
+     */
+    private final static ObjectIdInfo EMPTY = new ObjectIdInfo(PropertyName.NO_NAME, Object.class, null, false, null);
+
     public ObjectIdInfo(PropertyName name, Class<?> scope, Class<? extends ObjectIdGenerator<?>> gen,
             Class<? extends ObjectIdResolver> resolver)
     {
         this(name, scope, gen, false, resolver);
     }
 
-    @Deprecated // since 2.4
-    public ObjectIdInfo(PropertyName name, Class<?> scope, Class<? extends ObjectIdGenerator<?>> gen)
-    {
-        this(name, scope, gen, false);
-    }
-
-    @Deprecated // since 2.3
-    public ObjectIdInfo(String name, Class<?> scope, Class<? extends ObjectIdGenerator<?>> gen) {
-        this(new PropertyName(name), scope, gen, false);
-    }
-    
     protected ObjectIdInfo(PropertyName prop, Class<?> scope, Class<? extends ObjectIdGenerator<?>> gen,
             boolean alwaysAsId)
     {
@@ -56,11 +51,15 @@ public class ObjectIdInfo
         _resolver = resolver;
     }
 
+    public static ObjectIdInfo empty() {
+        return EMPTY;
+    }
+
     public ObjectIdInfo withAlwaysAsId(boolean state) {
         if (_alwaysAsId == state) {
             return this;
         }
-        return new ObjectIdInfo(_propertyName, _scope, _generator, state);
+        return new ObjectIdInfo(_propertyName, _scope, _generator, state, _resolver);
     }
     
     public PropertyName getPropertyName() { return _propertyName; }
@@ -72,8 +71,8 @@ public class ObjectIdInfo
     @Override
     public String toString() {
         return "ObjectIdInfo: propName="+_propertyName
-                +", scope="+(_scope == null ? "null" : _scope.getName())
-                +", generatorType="+(_generator == null ? "null" : _generator.getName())
+                +", scope="+ClassUtil.nameOf(_scope)
+                +", generatorType="+ClassUtil.nameOf(_generator)
                 +", alwaysAsId="+_alwaysAsId;
     }
 }
