@@ -21,10 +21,13 @@ public class AsArrayTypeDeserializer
     extends TypeDeserializerBase
     implements java.io.Serializable
 {
-    private static final long serialVersionUID = 5345570420394408290L;
+    private static final long serialVersionUID = 1L;
 
+    /**
+     * @since 2.8
+     */
     public AsArrayTypeDeserializer(JavaType bt, TypeIdResolver idRes,
-            String typePropertyName, boolean typeIdVisible, Class<?> defaultImpl)
+            String typePropertyName, boolean typeIdVisible, JavaType defaultImpl)
     {
         super(bt, idRes, typePropertyName, typeIdVisible, defaultImpl);
     }
@@ -80,57 +83,80 @@ public class AsArrayTypeDeserializer
      * deserialization.
      */
     @SuppressWarnings("resource")
-    private final Object _deserialize(JsonParser jp, DeserializationContext ctxt) throws IOException
+    protected Object _deserialize(JsonParser p, DeserializationContext ctxt) throws IOException
     {
         // 02-Aug-2013, tatu: May need to use native type ids
-        if (jp.canReadTypeId()) {
-            Object typeId = jp.getTypeId();
+        if (p.canReadTypeId()) {
+            Object typeId = p.getTypeId();
             if (typeId != null) {
-                return _deserializeWithNativeTypeId(jp, ctxt, typeId);
+                return _deserializeWithNativeTypeId(p, ctxt, typeId);
             }
         }
-        boolean hadStartArray = jp.isExpectedStartArrayToken();
-        String typeId = _locateTypeId(jp, ctxt);
+        boolean hadStartArray = p.isExpectedStartArrayToken();
+        String typeId = _locateTypeId(p, ctxt);
         JsonDeserializer<Object> deser = _findDeserializer(ctxt, typeId);
         // Minor complication: we may need to merge type id in?
-        if (_typeIdVisible && jp.getCurrentToken() == JsonToken.START_OBJECT) {
+        if (_typeIdVisible
+                // 06-Oct-2014, tatu: To fix [databind#408], must distinguish between
+                //   internal and external properties
+                //  TODO: but does it need to be injected in external case? Why not?
+                && !_usesExternalId()
+                && p.getCurrentToken() == JsonToken.START_OBJECT) {
             // but what if there's nowhere to add it in? Error? Or skip? For now, skip.
             TokenBuffer tb = new TokenBuffer(null, false);
             tb.writeStartObject(); // recreate START_OBJECT
             tb.writeFieldName(_typePropertyName);
             tb.writeString(typeId);
-            jp = JsonParserSequence.createFlattened(tb.asParser(jp), jp);
-            jp.nextToken();
+            // 02-Jul-2016, tatu: Depending on for JsonParserSequence is initialized it may
+            //   try to access current token; ensure there isn't one
+            p.clearCurrentToken();
+            p = JsonParserSequence.createFlattened(false, tb.asParser(p), p);
+            p.nextToken();
         }
-        Object value = deser.deserialize(jp, ctxt);
+        Object value = deser.deserialize(p, ctxt);
         // And then need the closing END_ARRAY
-        if (hadStartArray && jp.nextToken() != JsonToken.END_ARRAY) {
-            throw ctxt.wrongTokenException(jp, JsonToken.END_ARRAY,
+        if (hadStartArray && p.nextToken() != JsonToken.END_ARRAY) {
+            ctxt.reportWrongTokenException(baseType(), JsonToken.END_ARRAY,
                     "expected closing END_ARRAY after type information and deserialized value");
+            // 05-May-2016, tatu: Not 100% what to do if exception is stored for
+            //     future, and not thrown immediately: should probably skip until END_ARRAY
+
+            // ... but for now, fall through
         }
         return value;
     }    
     
-    protected final String _locateTypeId(JsonParser jp, DeserializationContext ctxt) throws IOException
+    protected String _locateTypeId(JsonParser p, DeserializationContext ctxt) throws IOException
     {
-        if (!jp.isExpectedStartArrayToken()) {
-            // [JACKSON-712] Need to allow even more customized handling, if something unexpected seen...
+        if (!p.isExpectedStartArrayToken()) {
+            // Need to allow even more customized handling, if something unexpected seen...
             // but should there be a way to limit this to likely success cases?
             if (_defaultImpl != null) {
                 return _idResolver.idFromBaseType();
             }
-            throw ctxt.wrongTokenException(jp, JsonToken.START_ARRAY, "need JSON Array to contain As.WRAPPER_ARRAY type information for class "+baseTypeName());
+             ctxt.reportWrongTokenException(baseType(), JsonToken.START_ARRAY,
+                     "need JSON Array to contain As.WRAPPER_ARRAY type information for class "+baseTypeName());
+             return null;
         }
         // And then type id as a String
-        JsonToken t = jp.nextToken();
+        JsonToken t = p.nextToken();
         if (t == JsonToken.VALUE_STRING) {
-            String result = jp.getText();
-            jp.nextToken();
+            String result = p.getText();
+            p.nextToken();
             return result;
         }
         if (_defaultImpl != null) {
             return _idResolver.idFromBaseType();
         }
-        throw ctxt.wrongTokenException(jp, JsonToken.VALUE_STRING, "need JSON String that contains type id (for subtype of "+baseTypeName()+")");
+        ctxt.reportWrongTokenException(baseType(), JsonToken.VALUE_STRING,
+                "need JSON String that contains type id (for subtype of %s)", baseTypeName());
+        return null;
+    }
+
+    /**
+     * @since 2.5
+     */
+    protected boolean _usesExternalId() {
+        return false;
     }
 }
