@@ -17,13 +17,16 @@ public final class NullNode
 
     public final static NullNode instance = new NullNode();
 
-    private NullNode() { }
+    /**
+     *<p>
+     * NOTE: visibility raised to `protected` in 2.9.3 to allow custom subtypes.
+     */
+    protected NullNode() { }
 
     public static NullNode getInstance() { return instance; }
 
     @Override
-    public JsonNodeType getNodeType()
-    {
+    public JsonNodeType getNodeType() {
         return JsonNodeType.NULL;
     }
 
@@ -42,15 +45,19 @@ public final class NullNode
     */
     
     @Override
-    public final void serialize(JsonGenerator jg, SerializerProvider provider)
-        throws IOException, JsonProcessingException
+    public final void serialize(JsonGenerator g, SerializerProvider provider)
+        throws IOException
     {
-        provider.defaultSerializeNull(jg);
+        provider.defaultSerializeNull(g);
     }
 
     @Override
-    public boolean equals(Object o)
-    {
+    public boolean equals(Object o) {
         return (o == this);
     }
+
+    @Override
+    public int hashCode() {
+        return JsonNodeType.NULL.ordinal();
+    }
 }
