@@ -3,13 +3,11 @@ package com.fasterxml.jackson.databind.ser.impl;
 import java.io.IOException;
 import java.lang.reflect.Type;
 
-import com.fasterxml.jackson.core.JsonGenerationException;
+import com.fasterxml.jackson.annotation.JsonFormat;
 import com.fasterxml.jackson.core.JsonGenerator;
-
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JacksonStdImpl;
 import com.fasterxml.jackson.databind.introspect.AnnotatedMember;
-import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonArrayFormatVisitor;
 import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatVisitorWrapper;
 import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatTypes;
 import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
@@ -22,6 +20,7 @@ import com.fasterxml.jackson.databind.type.TypeFactory;
  * Standard serializer used for <code>String[]</code> values.
  */
 @JacksonStdImpl
+@SuppressWarnings("serial")
 public class StringArraySerializer
     extends ArraySerializerBase<String[]>
     implements ContextualSerializer
@@ -29,10 +28,11 @@ public class StringArraySerializer
     /* Note: not clean in general, but we are betting against
      * anyone re-defining properties of String.class here...
      */
+    @SuppressWarnings("deprecation")
     private final static JavaType VALUE_TYPE = TypeFactory.defaultInstance().uncheckedSimpleType(String.class);
 
     public final static StringArraySerializer instance = new StringArraySerializer();
-    
+
     /**
      * Value serializer to use, if it's not the standard one
      * (if it is we can optimize serialization a lot)
@@ -46,17 +46,22 @@ public class StringArraySerializer
      */
     
     protected StringArraySerializer() {
-        super(String[].class, null);
+        super(String[].class);
         _elementSerializer = null;
     }
 
     @SuppressWarnings("unchecked")
     public StringArraySerializer(StringArraySerializer src,
-            BeanProperty prop, JsonSerializer<?> ser) {
-        super(src, prop);
+            BeanProperty prop, JsonSerializer<?> ser, Boolean unwrapSingle) {
+        super(src, prop, unwrapSingle);
         _elementSerializer = (JsonSerializer<Object>) ser;
     }
-    
+
+    @Override
+    public JsonSerializer<?> _withResolved(BeanProperty prop, Boolean unwrapSingle) {
+        return new StringArraySerializer(this, prop, _elementSerializer, unwrapSingle);
+    }
+
     /**
      * Strings never add type info; hence, even if type serializer is suggested,
      * we'll ignore it...
@@ -82,35 +87,39 @@ public class StringArraySerializer
          *    and it may have overrides
          */
         JsonSerializer<?> ser = null;
+
         // First: if we have a property, may have property-annotation overrides
         if (property != null) {
+            final AnnotationIntrospector ai = provider.getAnnotationIntrospector();
             AnnotatedMember m = property.getMember();
             if (m != null) {
-                Object serDef = provider.getAnnotationIntrospector().findContentSerializer(m);
+                Object serDef = ai.findContentSerializer(m);
                 if (serDef != null) {
                     ser = provider.serializerInstance(m, serDef);
                 }
             }
         }
+        // but since formats have both property overrides and global per-type defaults,
+        // need to do that separately
+        Boolean unwrapSingle = findFormatFeature(provider, property, String[].class,
+                JsonFormat.Feature.WRITE_SINGLE_ELEM_ARRAYS_UNWRAPPED);
         if (ser == null) {
             ser = _elementSerializer;
         }
-        // #124: May have a content converter
-        ser = findConvertingContentSerializer(provider, property, ser);
+        // May have a content converter
+        ser = findContextualConvertingSerializer(provider, property, ser);
         if (ser == null) {
             ser = provider.findValueSerializer(String.class, property);
-        } else {
-            ser = provider.handleSecondaryContextualization(ser, property);
         }
         // Optimization: default serializer just writes String, so we can avoid a call:
         if (isDefaultSerializer(ser)) {
             ser = null;
         }
         // note: will never have TypeSerializer, because Strings are "natural" type
-        if (ser == _elementSerializer) {
+        if ((ser == _elementSerializer) && (unwrapSingle == _unwrapSingle)) {
             return this;
         }
-        return new StringArraySerializer(this, property, ser);
+        return new StringArraySerializer(this, property, ser, unwrapSingle);
     }
 
     /*
@@ -130,8 +139,8 @@ public class StringArraySerializer
     }
     
     @Override
-    public boolean isEmpty(String[] value) {
-        return (value == null) || (value.length == 0);
+    public boolean isEmpty(SerializerProvider prov, String[] value) {
+        return (value.length == 0);
     }
 
     @Override
@@ -146,45 +155,54 @@ public class StringArraySerializer
      */
     
     @Override
-    public void serializeContents(String[] value, JsonGenerator jgen, SerializerProvider provider)
-        throws IOException, JsonGenerationException
+    public final void serialize(String[] value, JsonGenerator gen, SerializerProvider provider)
+        throws IOException
+    {
+        final int len = value.length;
+        if (len == 1) {
+            if (((_unwrapSingle == null) &&
+                    provider.isEnabled(SerializationFeature.WRITE_SINGLE_ELEM_ARRAYS_UNWRAPPED))
+                    || (_unwrapSingle == Boolean.TRUE)) {
+                serializeContents(value, gen, provider);
+                return;
+            }
+        }
+        gen.writeStartArray(len);
+        serializeContents(value, gen, provider);
+        gen.writeEndArray();
+    }
+    
+    @Override
+    public void serializeContents(String[] value, JsonGenerator gen, SerializerProvider provider)
+        throws IOException
     {
         final int len = value.length;
         if (len == 0) {
             return;
         }
         if (_elementSerializer != null) {
-            serializeContentsSlow(value, jgen, provider, _elementSerializer);
+            serializeContentsSlow(value, gen, provider, _elementSerializer);
             return;
         }
-        /* 08-Dec-2008, tatus: If we want this to be fully overridable
-         *  (for example, to support String cleanup during writing
-         *  or something), we should find serializer  by provider.
-         *  But for now, that seems like an overkill: and caller can
-         *  add custom serializer if that is needed as well.
-         * (ditto for null values)
-         */
-        //JsonSerializer<String> ser = (JsonSerializer<String>)provider.findValueSerializer(String.class);
         for (int i = 0; i < len; ++i) {
             String str = value[i];
             if (str == null) {
-                jgen.writeNull();
+                gen.writeNull();
             } else {
-                //ser.serialize(value[i], jgen, provider);
-                jgen.writeString(value[i]);
+                gen.writeString(value[i]);
             }
         }
     }
 
-    private void serializeContentsSlow(String[] value, JsonGenerator jgen, SerializerProvider provider, JsonSerializer<Object> ser)
-        throws IOException, JsonGenerationException
+    private void serializeContentsSlow(String[] value, JsonGenerator gen, SerializerProvider provider, JsonSerializer<Object> ser)
+        throws IOException
     {
         for (int i = 0, len = value.length; i < len; ++i) {
             String str = value[i];
             if (str == null) {
-                provider.defaultSerializeNull(jgen);
+                provider.defaultSerializeNull(gen);
             } else {
-                ser.serialize(value[i], jgen, provider);
+                ser.serialize(value[i], gen, provider);
             }
         }
     }
@@ -197,11 +215,6 @@ public class StringArraySerializer
     @Override
     public void acceptJsonFormatVisitor(JsonFormatVisitorWrapper visitor, JavaType typeHint) throws JsonMappingException
     {
-        if (visitor != null) {
-            JsonArrayFormatVisitor v2 = visitor.expectArrayFormat(typeHint);
-            if (v2 != null) {
-                v2.itemsFormat(JsonFormatTypes.STRING);
-            }
-        }
+        visitArrayFormat(visitor, typeHint, JsonFormatTypes.STRING);
     }
 }
