@@ -3,15 +3,17 @@ package com.fasterxml.jackson.databind.ser.impl;
 import java.util.Iterator;
 import java.util.Map.Entry;
 
-import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.JsonGenerator;
 import com.fasterxml.jackson.core.io.SerializedString;
 import com.fasterxml.jackson.databind.JavaType;
 import com.fasterxml.jackson.databind.JsonMappingException;
 import com.fasterxml.jackson.databind.JsonNode;
 import com.fasterxml.jackson.databind.JsonSerializer;
 import com.fasterxml.jackson.databind.SerializerProvider;
+import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatVisitorWrapper;
+import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonObjectFormatVisitor;
 import com.fasterxml.jackson.databind.node.ObjectNode;
-import com.fasterxml.jackson.databind.ser.*;
+import com.fasterxml.jackson.databind.ser.BeanPropertyWriter;
 import com.fasterxml.jackson.databind.util.NameTransformer;
 
 /**
@@ -21,25 +23,28 @@ import com.fasterxml.jackson.databind.util.NameTransformer;
  */
 public class UnwrappingBeanPropertyWriter
     extends BeanPropertyWriter
+    implements java.io.Serializable
 {
+    private static final long serialVersionUID = 1L;
+
     /**
      * Transformer used to add prefix and/or suffix for properties
      * of unwrapped POJO.
      */
     protected final NameTransformer _nameTransformer;
-    
+
     /*
     /**********************************************************
     /* Life-cycle
     /**********************************************************
      */
-    
+
     public UnwrappingBeanPropertyWriter(BeanPropertyWriter base, NameTransformer unwrapper) {
         super(base);
         _nameTransformer = unwrapper;
     }
 
-    private UnwrappingBeanPropertyWriter(UnwrappingBeanPropertyWriter base, NameTransformer transformer,
+    protected UnwrappingBeanPropertyWriter(UnwrappingBeanPropertyWriter base, NameTransformer transformer,
             SerializedString name) {
         super(base, name);
         _nameTransformer = transformer;
@@ -54,7 +59,17 @@ public class UnwrappingBeanPropertyWriter
         // important: combine transformers:
         transformer = NameTransformer.chainedTransformer(transformer, _nameTransformer);
     
-        return new UnwrappingBeanPropertyWriter(this, transformer, new SerializedString(newName));
+        return _new(transformer, new SerializedString(newName));
+    }
+
+    /**
+     * Overridable factory method used by sub-classes
+     *
+     * @since 2.6.0
+     */
+    protected UnwrappingBeanPropertyWriter _new(NameTransformer transformer, SerializedString newName)
+    {
+        return new UnwrappingBeanPropertyWriter(this, transformer, newName);
     }
 
     /*
@@ -67,9 +82,9 @@ public class UnwrappingBeanPropertyWriter
     public boolean isUnwrapping() {
         return true;
     }
-    
+
     @Override
-    public void serializeAsField(Object bean, JsonGenerator jgen, SerializerProvider prov)
+    public void serializeAsField(Object bean, JsonGenerator gen, SerializerProvider prov)
             throws Exception
     {
         final Object value = get(bean);
@@ -89,7 +104,7 @@ public class UnwrappingBeanPropertyWriter
         }
         if (_suppressableValue != null) {
             if (MARKER_FOR_EMPTY == _suppressableValue) {
-                if (ser.isEmpty(value)) {
+                if (ser.isEmpty(prov, value)) {
                     return;
                 }
             } else if (_suppressableValue.equals(value)) {
@@ -98,20 +113,20 @@ public class UnwrappingBeanPropertyWriter
         }
         // For non-nulls, first: simple check for direct cycles
         if (value == bean) {
-            if (_handleSelfReference(bean, jgen, prov, ser)) {
+            if (_handleSelfReference(bean, gen, prov, ser)) {
                 return;
             }
         }
 
         // note: must verify we are using unwrapping serializer; if not, will write field name
         if (!ser.isUnwrappingSerializer()) {
-            jgen.writeFieldName(_name);
+            gen.writeFieldName(_name);
         }
 
         if (_typeSerializer == null) {
-            ser.serialize(value, jgen, prov);
+            ser.serialize(value, gen, prov);
         } else {
-            ser.serializeWithType(value, jgen, prov, _typeSerializer);
+            ser.serializeWithType(value, gen, prov, _typeSerializer);
         }
     }
 
@@ -119,14 +134,14 @@ public class UnwrappingBeanPropertyWriter
     @Override
     public void assignSerializer(JsonSerializer<Object> ser)
     {
-        super.assignSerializer(ser);
-        if (_serializer != null) {
+        if (ser != null) {
             NameTransformer t = _nameTransformer;
-            if (_serializer.isUnwrappingSerializer()) {
-                t = NameTransformer.chainedTransformer(t, ((UnwrappingBeanSerializer) _serializer)._nameTransformer);
+            if (ser.isUnwrappingSerializer()) {
+                t = NameTransformer.chainedTransformer(t, ((UnwrappingBeanSerializer) ser)._nameTransformer);
             }
-            _serializer = _serializer.unwrappingSerializer(t);
+            ser = ser.unwrappingSerializer(t);
         }
+        super.assignSerializer(ser);
     }
 
     /*
@@ -135,22 +150,29 @@ public class UnwrappingBeanPropertyWriter
     /**********************************************************
      */
 
-    // TODO: if/how to change this? Base class has this
-/*    
     @Override
-    public void depositSchemaProperty(JsonObjectFormatVisitor objectVisitor)
-        throws JsonMappingException
+    public void depositSchemaProperty(final JsonObjectFormatVisitor visitor,
+            SerializerProvider provider) throws JsonMappingException
     {
-        if (objectVisitor != null) {
-            if (isRequired()) {
-                objectVisitor.property(this); 
-            } else {
-                objectVisitor.optionalProperty(this);
-            }
+        JsonSerializer<Object> ser = provider
+                .findValueSerializer(this.getType(), this)
+                .unwrappingSerializer(_nameTransformer);
+
+        if (ser.isUnwrappingSerializer()) {
+            ser.acceptJsonFormatVisitor(new JsonFormatVisitorWrapper.Base(provider) {
+                // an unwrapping serializer will always expect ObjectFormat,
+                // hence, the other cases do not have to be implemented
+                @Override
+                public JsonObjectFormatVisitor expectObjectFormat(JavaType type)
+                        throws JsonMappingException {
+                    return visitor;
+                }
+            }, this.getType());
+        } else {
+            super.depositSchemaProperty(visitor, provider);
         }
     }
-    */
-    
+
     // Override needed to support legacy JSON Schema generator
     @Override
     protected void _depositSchemaProperty(ObjectNode propertiesNode, JsonNode schemaNode)
