@@ -1,8 +1,6 @@
 package com.fasterxml.jackson.databind.ser.impl;
 
-import com.fasterxml.jackson.core.JsonGenerationException;
 import com.fasterxml.jackson.core.JsonGenerator;
-
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.jsontype.TypeSerializer;
 import com.fasterxml.jackson.databind.ser.BeanPropertyWriter;
@@ -10,10 +8,14 @@ import com.fasterxml.jackson.databind.ser.std.BeanSerializerBase;
 import com.fasterxml.jackson.databind.util.NameTransformer;
 
 import java.io.IOException;
+import java.util.Set;
 
 public class UnwrappingBeanSerializer
     extends BeanSerializerBase
+    implements java.io.Serializable
 {
+    private static final long serialVersionUID = 1L;
+
     /**
      * Transformer used to add prefix and/or suffix for properties
      * of unwrapped POJO.
@@ -46,12 +48,12 @@ public class UnwrappingBeanSerializer
         super(src, objectIdWriter, filterId);
         _nameTransformer = src._nameTransformer;
     }
-    
-    protected UnwrappingBeanSerializer(UnwrappingBeanSerializer src, String[] toIgnore) {
+
+    protected UnwrappingBeanSerializer(UnwrappingBeanSerializer src, Set<String> toIgnore) {
         super(src, toIgnore);
         _nameTransformer = src._nameTransformer;
     }
-    
+
     /*
     /**********************************************************
     /* Life-cycle: factory methods, fluent factories
@@ -75,17 +77,17 @@ public class UnwrappingBeanSerializer
     }
 
     @Override
-    protected BeanSerializerBase withFilterId(Object filterId) {
+    public BeanSerializerBase withFilterId(Object filterId) {
         return new UnwrappingBeanSerializer(this, _objectIdWriter, filterId);
     }
-    
+
     @Override
-    protected BeanSerializerBase withIgnorals(String[] toIgnore) {
+    protected BeanSerializerBase withIgnorals(Set<String> toIgnore) {
         return new UnwrappingBeanSerializer(this, toIgnore);
     }
 
     /**
-     * JSON Array output can not be done if unwrapping operation is
+     * JSON Array output cannot be done if unwrapping operation is
      * requested; so implementation will simply return 'this'.
      */
     @Override
@@ -105,37 +107,37 @@ public class UnwrappingBeanSerializer
      * {@link BeanPropertyWriter} instances.
      */
     @Override
-    public final void serialize(Object bean, JsonGenerator jgen, SerializerProvider provider)
-        throws IOException, JsonGenerationException
+    public final void serialize(Object bean, JsonGenerator gen, SerializerProvider provider) throws IOException
     {
+        gen.setCurrentValue(bean); // [databind#631]
         if (_objectIdWriter != null) {
-            _serializeWithObjectId(bean, jgen, provider, false);
+            _serializeWithObjectId(bean, gen, provider, false);
             return;
         }
         if (_propertyFilterId != null) {
-            serializeFieldsFiltered(bean, jgen, provider);
+            serializeFieldsFiltered(bean, gen, provider);
         } else {
-            serializeFields(bean, jgen, provider);
+            serializeFields(bean, gen, provider);
         }
     }
     
     @Override
-    public void serializeWithType(Object bean, JsonGenerator jgen, SerializerProvider provider, TypeSerializer typeSer)
-        throws IOException, JsonGenerationException
+    public void serializeWithType(Object bean, JsonGenerator gen, SerializerProvider provider,
+    		TypeSerializer typeSer) throws IOException
     {
         if (provider.isEnabled(SerializationFeature.FAIL_ON_UNWRAPPED_TYPE_IDENTIFIERS)) {
-            throw new JsonMappingException("Unwrapped property requires use of type information: can not serialize without disabling `SerializationFeature.FAIL_ON_UNWRAPPED_TYPE_IDENTIFIERS`");
+            provider.reportBadDefinition(handledType(),
+                    "Unwrapped property requires use of type information: cannot serialize without disabling `SerializationFeature.FAIL_ON_UNWRAPPED_TYPE_IDENTIFIERS`");
         }
-
+        gen.setCurrentValue(bean); // [databind#631]
         if (_objectIdWriter != null) {
-            _serializeWithObjectId(bean, jgen, provider, typeSer);
+            _serializeWithObjectId(bean, gen, provider, typeSer);
             return;
         }
-
         if (_propertyFilterId != null) {
-            serializeFieldsFiltered(bean, jgen, provider);
+            serializeFieldsFiltered(bean, gen, provider);
         } else {
-            serializeFields(bean, jgen, provider);
+            serializeFields(bean, gen, provider);
         }
     }
 
