@@ -4,17 +4,19 @@ import java.io.*;
 import java.nio.ByteBuffer;
 
 import com.fasterxml.jackson.core.*;
-
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonArrayFormatVisitor;
+import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatTypes;
+import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatVisitorWrapper;
 import com.fasterxml.jackson.databind.util.ByteBufferBackedInputStream;
 
+@SuppressWarnings("serial")
 public class ByteBufferSerializer extends StdScalarSerializer<ByteBuffer>
 {
     public ByteBufferSerializer() { super(ByteBuffer.class); }
 
     @Override
-    public void serialize(ByteBuffer bbuf, JsonGenerator gen, SerializerProvider provider)
-        throws IOException, JsonGenerationException
+    public void serialize(ByteBuffer bbuf, JsonGenerator gen, SerializerProvider provider) throws IOException
     {
         // first, simple case when wrapping an array...
         if (bbuf.hasArray()) {
@@ -31,4 +33,15 @@ public class ByteBufferSerializer extends StdScalarSerializer<ByteBuffer>
         gen.writeBinary(in, copy.remaining());
         in.close();
     }
+
+    @Override // since 2.9
+    public void acceptJsonFormatVisitor(JsonFormatVisitorWrapper visitor, JavaType typeHint)
+        throws JsonMappingException
+    {
+        // 31-Mar-2017, tatu: Use same type as `ByteArraySerializer`: not optimal but has to do
+        JsonArrayFormatVisitor v2 = visitor.expectArrayFormat(typeHint);
+        if (v2 != null) {
+            v2.itemsFormat(JsonFormatTypes.INTEGER);
+        }
+    }
 }
