@@ -3,8 +3,8 @@ package com.fasterxml.jackson.databind.ser.std;
 import java.io.IOException;
 import java.lang.reflect.Type;
 
-import com.fasterxml.jackson.core.JsonGenerationException;
 import com.fasterxml.jackson.core.JsonGenerator;
+
 import com.fasterxml.jackson.databind.JavaType;
 import com.fasterxml.jackson.databind.JsonMappingException;
 import com.fasterxml.jackson.databind.JsonNode;
@@ -15,16 +15,16 @@ import com.fasterxml.jackson.databind.jsonFormatVisitors.JsonFormatVisitorWrappe
  * Also: default bean access will not do much good with Class.class. But
  * we can just serialize the class name and that should be enough.
  */
+@SuppressWarnings("serial")
 public class ClassSerializer
     extends StdScalarSerializer<Class<?>>
 {
     public ClassSerializer() { super(Class.class, false); }
 
     @Override
-    public void serialize(Class<?> value, JsonGenerator jgen, SerializerProvider provider)
-        throws IOException, JsonGenerationException
+    public void serialize(Class<?> value, JsonGenerator g, SerializerProvider provider) throws IOException
     {
-        jgen.writeString(value.getName());
+        g.writeString(value.getName());
     }
 
     @Override
@@ -35,8 +35,8 @@ public class ClassSerializer
     
     @Override
     public void acceptJsonFormatVisitor(JsonFormatVisitorWrapper visitor, JavaType typeHint)
-            throws JsonMappingException
+        throws JsonMappingException
     {
-        visitor.expectStringFormat(typeHint);
+        visitStringFormat(visitor, typeHint);
     }
 }
\ No newline at end of file
