@@ -3,25 +3,29 @@ package com.fasterxml.jackson.databind.convert;
 import java.util.LinkedHashMap;
 import java.util.Map;
 
+import com.fasterxml.jackson.core.JsonParser;
 import com.fasterxml.jackson.core.JsonProcessingException;
 import com.fasterxml.jackson.core.TreeNode;
 import com.fasterxml.jackson.databind.*;
+import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
 import com.fasterxml.jackson.databind.annotation.JsonSerialize;
 import com.fasterxml.jackson.databind.node.ObjectNode;
 import com.fasterxml.jackson.databind.util.StdConverter;
 
+/**
+ * Tests for various conversions, especially ones using
+ * {@link ObjectMapper#convertValue(Object, Class)}.
+ */
 public class TestBeanConversions
     extends com.fasterxml.jackson.databind.BaseMapTest
 {
-    final ObjectMapper MAPPER = new ObjectMapper();
-
-    static class Point {
+    static class PointZ {
         public int x, y;
 
         public int z = -13;
 
-        public Point() { }
-        public Point(int a, int b, int c)
+        public PointZ() { }
+        public PointZ(int a, int b, int c)
         {
             x = a;
             y = b;
@@ -64,7 +68,7 @@ public class TestBeanConversions
         public Leaf(int v) { value = v; }
     }
     
-    // [Issue#288]
+    // [databind#288]
 
     @JsonSerialize(converter = ConvertingBeanConverter.class)
     static class ConvertingBean { 
@@ -90,6 +94,23 @@ public class TestBeanConversions
           return new DummyBean(cb.x, cb.y);
        }
     }
+    
+    @JsonDeserialize(using = NullBeanDeserializer.class)
+    static class NullBean {
+        public static final NullBean NULL_INSTANCE = new NullBean();
+    }
+    
+    static class NullBeanDeserializer extends JsonDeserializer<NullBean> {
+        @Override
+        public NullBean getNullValue(final DeserializationContext context) {
+            return NullBean.NULL_INSTANCE;
+        }
+        
+        @Override
+        public NullBean deserialize(final JsonParser parser, final DeserializationContext context) {
+            throw new UnsupportedOperationException();
+        }
+    }
 
     /*
     /**********************************************************
@@ -97,11 +118,13 @@ public class TestBeanConversions
     /**********************************************************
      */
     
+    private final ObjectMapper MAPPER = new ObjectMapper();
+
     public void testBeanConvert()
     {
         // should have no problems convert between compatible beans...
         PointStrings input = new PointStrings("37", "-9");
-        Point point = MAPPER.convertValue(input, Point.class);
+        PointZ point = MAPPER.convertValue(input, PointZ.class);
         assertEquals(37, point.x);
         assertEquals(-9, point.y);
         // z not included in input, will be whatever default constructor provides
@@ -124,7 +147,7 @@ public class TestBeanConversions
         try {
             MAPPER.readValue("{\"boolProp\":\"foobar\"}", BooleanBean.class);
         } catch (JsonMappingException e) {
-            verifyException(e, "from String value 'foobar'");
+            verifyException(e, "Cannot deserialize value of type `boolean` from String");
         }
     }
 
@@ -137,7 +160,7 @@ public class TestBeanConversions
         assertEquals("foo", a2.getData());
     }
 
-    // [JACKSON-710]: should work regardless of wrapping...
+    // should work regardless of wrapping...
     public void testWrapping() throws Exception
     {
         ObjectMapper wrappingMapper = new ObjectMapper();
@@ -182,15 +205,15 @@ public class TestBeanConversions
     
     private void _convertAndVerifyPoint(ObjectMapper m)
     {
-        final Point input = new Point(1, 2, 3);
-        Point output = m.convertValue(input, Point.class);
+        final PointZ input = new PointZ(1, 2, 3);
+        PointZ output = m.convertValue(input, PointZ.class);
         assertEquals(1, output.x);
         assertEquals(2, output.y);
         assertEquals(3, output.z);
     }
 
     /**
-     * Need to test "shortcuts" introduced by [Issue-11]
+     * Need to test "shortcuts" introduced by [databind#11]
      */
     public void testIssue11() throws Exception
     {
@@ -252,5 +275,15 @@ public class TestBeanConversions
         String json = MAPPER.writeValueAsString(new ConvertingBean(1, 2));
         // must be  {"a":2,"b":4}
         assertEquals("{\"a\":2,\"b\":4}", json);
-     }
+    }
+    
+    // Test null conversions from [databind#1433]
+    public void testConversionIssue1433() throws Exception
+    {
+        assertNull(MAPPER.convertValue(null, Object.class));
+        assertNull(MAPPER.convertValue(null, PointZ.class));
+        
+        assertSame(NullBean.NULL_INSTANCE,
+                MAPPER.convertValue(null, NullBean.class));
+    }
 }
