@@ -1,20 +1,17 @@
 package com.fasterxml.jackson.databind.deser;
 
 import java.io.*;
-import java.lang.annotation.ElementType;
-import java.lang.annotation.Retention;
-import java.lang.annotation.RetentionPolicy;
-import java.lang.annotation.Target;
+import java.lang.annotation.*;
 import java.util.*;
 
 import com.fasterxml.jackson.annotation.JsonCreator;
 import com.fasterxml.jackson.annotation.JsonProperty;
+
 import com.fasterxml.jackson.core.*;
+
 import com.fasterxml.jackson.databind.*;
-import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
-import com.fasterxml.jackson.databind.annotation.JsonSerialize;
-import com.fasterxml.jackson.databind.deser.std.StdDelegatingDeserializer;
-import com.fasterxml.jackson.databind.deser.std.StdDeserializer;
+import com.fasterxml.jackson.databind.annotation.*;
+import com.fasterxml.jackson.databind.deser.std.*;
 import com.fasterxml.jackson.databind.module.SimpleModule;
 import com.fasterxml.jackson.databind.util.StdConverter;
 
@@ -42,11 +39,11 @@ public class TestCustomDeserializers
         }
 
         @Override
-        public T deserialize(JsonParser jp, DeserializationContext ctxt)
-            throws IOException, JsonProcessingException
+        public T deserialize(JsonParser p, DeserializationContext ctxt)
+            throws IOException
         {
             // need to skip, if structured...
-            jp.skipChildren();
+            p.skipChildren();
             return value;
         }
     }
@@ -70,29 +67,29 @@ public class TestCustomDeserializers
     static class CustomBeanDeserializer extends JsonDeserializer<CustomBean>
     {
         @Override
-        public CustomBean deserialize(JsonParser jp, DeserializationContext ctxt) throws IOException
+        public CustomBean deserialize(JsonParser p, DeserializationContext ctxt) throws IOException
         {
             int a = 0, b = 0;
-            JsonToken t = jp.getCurrentToken();
+            JsonToken t = p.getCurrentToken();
             if (t == JsonToken.START_OBJECT) {
-                t = jp.nextToken();
+                t = p.nextToken();
             } else if (t != JsonToken.FIELD_NAME) {
                 throw new Error();
             }
             while(t == JsonToken.FIELD_NAME) {
-                final String fieldName = jp.getCurrentName();
-                t = jp.nextToken();
+                final String fieldName = p.getCurrentName();
+                t = p.nextToken();
                 if (t != JsonToken.VALUE_NUMBER_INT) {
-                    throw new JsonParseException("expecting number got "+ t, jp.getCurrentLocation());
+                    throw new JsonParseException(p, "expecting number got "+ t);
                 }
                 if (fieldName.equals("a")) {
-                    a = jp.getIntValue();
+                    a = p.getIntValue();
                 } else if (fieldName.equals("b")) {
-                    b = jp.getIntValue();
+                    b = p.getIntValue();
                 } else {
                     throw new Error();
                 }
-                t = jp.nextToken();
+                t = p.nextToken();
             }
             return new CustomBean(a, b);
         }
@@ -107,7 +104,6 @@ public class TestCustomDeserializers
         }
     }
 
-    // [JACKSON-882]
     public static class CustomKey {
         private final int id;
 
@@ -135,8 +131,8 @@ public class TestCustomDeserializers
      
     static class CustomKeySerializer extends JsonSerializer<CustomKey> {
         @Override
-        public void serialize(CustomKey value, JsonGenerator jgen, SerializerProvider provider) throws IOException {
-            jgen.writeFieldName(String.valueOf(value.getId()));
+        public void serialize(CustomKey value, JsonGenerator g, SerializerProvider provider) throws IOException {
+            g.writeFieldName(String.valueOf(value.getId()));
         }
     }
 
@@ -147,7 +143,7 @@ public class TestCustomDeserializers
         }
     }
 
-    // [#375]
+    // [databind#375]
 
     @Target({ElementType.FIELD})
     @Retention(RetentionPolicy.RUNTIME)
@@ -206,9 +202,9 @@ public class TestCustomDeserializers
         }
 
         @Override
-        public Bean375Inner deserialize(JsonParser jp, DeserializationContext ctxt)
+        public Bean375Inner deserialize(JsonParser p, DeserializationContext ctxt)
                 throws IOException, JsonProcessingException {
-            int x = jp.getIntValue();
+            int x = p.getIntValue();
             if (negative) {
                 x = -x;
             } else {
@@ -228,9 +224,89 @@ public class TestCustomDeserializers
             }
             return this;
         }
-        
     }
-    
+
+    // for [databind#631]
+    static class Issue631Bean
+    {
+        @JsonDeserialize(using=ParentClassDeserializer.class)
+        public Object prop;
+    }
+
+    static class ParentClassDeserializer
+        extends StdScalarDeserializer<Object>
+    {
+        protected ParentClassDeserializer() {
+            super(Object.class);
+        }
+
+        @Override
+        public Object deserialize(JsonParser p, DeserializationContext ctxt)
+                throws IOException {
+            Object parent = p.getCurrentValue();
+            String desc = (parent == null) ? "NULL" : parent.getClass().getSimpleName();
+            return "prop/"+ desc;
+        }
+    }
+
+    static class UCStringDeserializer extends StdDeserializer<String> {
+        public UCStringDeserializer() { super(String.class); }
+
+        @Override
+        public String deserialize(JsonParser p, DeserializationContext ctxt)
+                throws IOException {
+            return p.getText().toUpperCase();
+        }
+    }
+
+    static class DelegatingModuleImpl extends SimpleModule
+    {
+        public DelegatingModuleImpl() {
+            super("test", Version.unknownVersion());
+        }
+
+        @Override
+        public void setupModule(SetupContext context)
+        {
+            super.setupModule(context);
+            context.addBeanDeserializerModifier(new BeanDeserializerModifier() {
+                @Override
+                public JsonDeserializer<?> modifyDeserializer(DeserializationConfig config,
+                        BeanDescription beanDesc, JsonDeserializer<?> deserializer) {
+                    if (deserializer.handledType() == String.class) {
+                        JsonDeserializer<?> d = new MyStringDeserializer(deserializer);
+                        // just for test coverage purposes...
+                        if (d.getDelegatee() != deserializer) {
+                            throw new Error("Cannot access delegatee!");
+                        }
+                        return d;
+                    }
+                    return deserializer;
+                }
+            });
+        }
+    }
+
+    static class MyStringDeserializer extends DelegatingDeserializer
+    {
+        public MyStringDeserializer(JsonDeserializer<?> newDel) {
+            super(newDel);
+        }
+
+        @Override
+        protected JsonDeserializer<?> newDelegatingInstance(JsonDeserializer<?> newDel) {
+            return new MyStringDeserializer(newDel);
+        }
+
+        @Override
+        public Object deserialize(JsonParser p, DeserializationContext ctxt)
+            throws IOException
+        {
+            Object ob = _delegatee.deserialize(p, ctxt);
+            return "MY:"+ob;
+        }
+    }
+
     /*
     /**********************************************************
     /* Unit tests
@@ -303,6 +379,26 @@ public class TestCustomDeserializers
         assertEquals(7, imm.y);
     }
 
+    // [databind#623]
+    public void testJsonNodeDelegating() throws Exception
+    {
+        ObjectMapper mapper = new ObjectMapper();
+        SimpleModule module = new SimpleModule("test", Version.unknownVersion());
+        module.addDeserializer(Immutable.class,
+            new StdNodeBasedDeserializer<Immutable>(Immutable.class) {
+                @Override
+                public Immutable convert(JsonNode root, DeserializationContext ctxt) throws IOException {
+                    int x = root.path("x").asInt();
+                    int y = root.path("y").asInt();
+                    return new Immutable(x, y);
+                }
+        });
+        mapper.registerModule(module);
+        Immutable imm = mapper.readValue("{\"x\":-10,\"y\":3}", Immutable.class);
+        assertEquals(-10, imm.x);
+        assertEquals(3, imm.y);
+    }
+    
     public void testIssue882() throws Exception
     {
         Model original = new Model(Collections.singletonMap(new CustomKey(123), "test"));
@@ -332,4 +428,32 @@ public class TestCustomDeserializers
         assertNotNull(w.value.inner);
         assertEquals(-13, w.value.inner.x);
     }
+
+    // [#631]: "current value" access
+    public void testCurrentValueAccess() throws Exception
+    {
+        Issue631Bean bean = MAPPER.readValue(aposToQuotes("{'prop':'stuff'}"),
+                Issue631Bean.class);
+        assertNotNull(bean);
+        assertEquals("prop/Issue631Bean", bean.prop);
+    }
+
+    public void testCustomStringDeser() throws Exception
+    {
+        ObjectMapper mapper = new ObjectMapper().registerModule(
+                new SimpleModule().addDeserializer(String.class, new UCStringDeserializer())
+                );
+        assertEquals("FOO", mapper.readValue(quote("foo"), String.class));
+        StringWrapper sw = mapper.readValue("{\"str\":\"foo\"}", StringWrapper.class);
+        assertNotNull(sw);
+        assertEquals("FOO", sw.str);
+    }
+
+    public void testDelegatingDeserializer() throws Exception
+    {
+        ObjectMapper mapper = new ObjectMapper().registerModule(
+                new DelegatingModuleImpl());
+        String str = mapper.readValue(quote("foo"), String.class);
+        assertEquals("MY:foo", str);
+    }
 }
