@@ -11,12 +11,6 @@ import com.fasterxml.jackson.databind.util.TokenBuffer;
 public class TestDefaultForObject
     extends BaseMapTest
 {
-    /*
-    /**********************************************************
-    /* Helper types
-    /**********************************************************
-     */
-
     static abstract class AbstractBean { }
     
     static class StringBean extends AbstractBean { // ha, punny!
@@ -32,17 +26,16 @@ public class TestDefaultForObject
      * Another enum type, but this time forcing sub-classing
      */
     enum ComplexChoice {
-    	MAYBE(true), PROBABLY_NOT(false);
+        MAYBE(true), PROBABLY_NOT(false);
 
-    	private boolean state;
+        private boolean state;
     	
-    	private ComplexChoice(boolean b) { state = b; }
+        private ComplexChoice(boolean b) { state = b; }
     	
         @Override
-    	public String toString() { return String.valueOf(state); }
+        public String toString() { return String.valueOf(state); }
     }
 
-    // [JACKSON-311]
     static class PolymorphicType {
         public String foo;
         public Object bar;
@@ -70,7 +63,6 @@ public class TestDefaultForObject
         public ObjectHolder(Object v) { value = v; }
     }
 
-    // [JACKSON-352]
     static class DomainBean {
         public int weight;
     }
@@ -113,6 +105,22 @@ public class TestDefaultForObject
         assertEquals("abc", ((StringBean) result[0]).name);
     }
 
+    // with 2.5, another test to check that "as-property" is valid option
+    public void testBeanAsObjectUsingAsProperty() throws Exception
+    {
+        ObjectMapper m = new ObjectMapper();
+        m.enableDefaultTypingAsProperty(ObjectMapper.DefaultTyping.NON_FINAL,
+                ".hype");
+        // note: need to wrap, to get declared as Object
+        String json = m.writeValueAsString(new StringBean("abc"));
+        
+        // Ok: serialization seems to work as expected. Now deserialize:
+        Object result = m.readValue(json, Object.class);
+        assertNotNull(result);
+        assertEquals(StringBean.class, result.getClass());
+        assertEquals("abc", ((StringBean) result).name);
+    }
+    
     /**
      * Unit test that verifies that an abstract bean is stored with type information
      * if default type information is enabled for non-concrete types.
@@ -128,7 +136,7 @@ public class TestDefaultForObject
             fail("Should have failed");
         } catch (JsonMappingException e) {
             // let's use whatever is currently thrown exception... may change tho
-            verifyException(e, "can not construct");
+            verifyException(e, "cannot construct");
         }
         
         // and then that we will succeed with default type info
@@ -331,6 +339,18 @@ public class TestDefaultForObject
         String json = mapper.writeValueAsString(new BeanHolder(new StringBean("punny")));
         assertEquals("{\"bean\":{\"*CLASS*\":\"com.fasterxml.jackson.databind.jsontype.TestDefaultForObject$StringBean\",\"name\":\"punny\"}}", json);
     }
+
+    public void testNoGoWithExternalProperty() throws Exception
+    {
+        ObjectMapper mapper = new ObjectMapper();
+        try {
+            mapper.enableDefaultTyping(ObjectMapper.DefaultTyping.JAVA_LANG_OBJECT,
+                    JsonTypeInfo.As.EXTERNAL_PROPERTY);
+            fail("Should not have passed");
+        } catch (IllegalArgumentException e) {
+            verifyException(e, "Cannot use includeAs of EXTERNAL_PROPERTY");
+        }
+    }
     
     /*
     /**********************************************************
