@@ -8,7 +8,9 @@ import static org.junit.Assert.*;
 
 import org.junit.Assert;
 
+import com.fasterxml.jackson.annotation.JsonTypeInfo;
 import com.fasterxml.jackson.core.*;
+import com.fasterxml.jackson.core.type.WritableTypeId;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.annotation.JsonDeserialize;
 import com.fasterxml.jackson.databind.annotation.JsonSerialize;
@@ -31,14 +33,11 @@ public class TestConversions extends BaseMapTest
         public Leaf() { }
         public Leaf(int v) { value = v; }
     }
-    
-    // MixIn for [JACKSON-554]
-    @JsonDeserialize(using = LeafDeserializer.class)
-    public static class LeafMixIn
-    {
-    }
 
-    // for [Issue#467]
+    @JsonDeserialize(using = LeafDeserializer.class)
+    public static class LeafMixIn { }
+
+    // for [databind#467]
     @JsonSerialize(using=Issue467Serializer.class)
     static class Issue467Bean  {
         public int i;
@@ -72,7 +71,16 @@ public class TestConversions extends BaseMapTest
 
         public Issue467TmpBean(int i) { x = i; }
     }
-    
+
+    static class Issue709Bean {
+        public byte[] data;
+    }
+
+    @JsonTypeInfo(use=JsonTypeInfo.Id.CLASS, include=JsonTypeInfo.As.PROPERTY, property="_class")
+    static class LongContainer1940 {
+        public Long longObj;
+    }
+
     /*
     /**********************************************************
     /* Unit tests
@@ -121,12 +129,11 @@ public class TestConversions extends BaseMapTest
         }
     }
 
-    // Test for [JACKSON-554]
     public void testTreeToValue() throws Exception
     {
         String JSON = "{\"leaf\":{\"value\":13}}";
         ObjectMapper mapper = new ObjectMapper();
-        mapper.addMixInAnnotations(Leaf.class, LeafMixIn.class);
+        mapper.addMixIn(Leaf.class, LeafMixIn.class);
         JsonNode root = mapper.readTree(JSON);
         // Ok, try converting to bean using two mechanisms
         Root r1 = mapper.treeToValue(root, Root.class);
@@ -134,7 +141,17 @@ public class TestConversions extends BaseMapTest
         assertEquals(13, r1.leaf.value);
     }
 
-    // Test for [JACKSON-631]
+    // [databind#1208]: should coerce POJOs at least at root level
+    public void testTreeToValueWithPOJO() throws Exception
+    {
+        Calendar c = Calendar.getInstance();
+        c.setTime(new java.util.Date(0));
+        ValueNode pojoNode = MAPPER.getNodeFactory().pojoNode(c);        
+        Calendar result = MAPPER.treeToValue(pojoNode, Calendar.class);
+        assertNotNull(result);
+        assertEquals(result.getTimeInMillis(), c.getTimeInMillis());
+    }
+
     public void testBase64Text() throws Exception
     {
         // let's actually iterate over sets of encoding modes, lengths
@@ -166,10 +183,6 @@ public class TestConversions extends BaseMapTest
         }
     }
 
-    static class Issue709Bean {
-        public byte[] data;
-    }
-    
     /**
      * Simple test to verify that byte[] values can be handled properly when
      * converting, as long as there is metadata (from POJO definitions).
@@ -202,11 +215,11 @@ public class TestConversions extends BaseMapTest
         assertEquals(3, data.length);
     }
 
-    // [Issue#232]
+    // [databind#232]
     public void testBigDecimalAsPlainStringTreeConversion() throws Exception
     {
         ObjectMapper mapper = new ObjectMapper();
-        mapper.enable(SerializationFeature.WRITE_BIGDECIMAL_AS_PLAIN);
+        mapper.enable(JsonGenerator.Feature.WRITE_BIGDECIMAL_AS_PLAIN);
         Map<String, Object> map = new HashMap<String, Object>();
         String PI_STR = "3.00000000";
         map.put("pi", new BigDecimal(PI_STR));
@@ -231,15 +244,17 @@ public class TestConversions extends BaseMapTest
         }
 
         @Override
-        public void serializeWithType(JsonGenerator jgen,
-                SerializerProvider provider, TypeSerializer typeSer) throws IOException {
-            typeSer.writeTypePrefixForObject(this, jgen);
-            serialize(jgen, provider);
-            typeSer.writeTypeSuffixForObject(this, jgen);
+        public void serializeWithType(JsonGenerator g,
+                SerializerProvider provider, TypeSerializer typeSer) throws IOException
+        {
+            WritableTypeId typeIdDef = new WritableTypeId(this, JsonToken.START_OBJECT);
+            typeSer.writeTypePrefix(g, typeIdDef);
+            serialize(g, provider);
+            typeSer.writeTypePrefix(g, typeIdDef);
         }    
     }
 
-    // [#433]
+    // [databind#433]
     public void testBeanToTree() throws Exception
     {
         final CustomSerializedPojo pojo = new CustomSerializedPojo();
@@ -248,7 +263,7 @@ public class TestConversions extends BaseMapTest
         assertEquals(JsonNodeType.OBJECT, node.getNodeType());
     }
 
-    // [#467]
+    // [databind#467]
     public void testConversionOfPojos() throws Exception
     {
         final Issue467Bean input = new Issue467Bean(13);
@@ -264,7 +279,7 @@ public class TestConversions extends BaseMapTest
         assertEquals(EXP, MAPPER.writeValueAsString(tree));
     }
 
-    // [#467]
+    // [databind#467]
     public void testConversionOfTrees() throws Exception
     {
         final Issue467Tree input = new Issue467Tree();
@@ -279,5 +294,12 @@ public class TestConversions extends BaseMapTest
         assertTrue("Expected Object, got "+tree.getNodeType(), tree.isBoolean());
         assertEquals(EXP, MAPPER.writeValueAsString(tree));
     }
-}
 
+    // [databind#1940]: losing of precision due to coercion
+    public void testBufferedLongViaCoercion() throws Exception {
+        long EXP = 1519348261000L;
+        JsonNode tree = MAPPER.readTree("{\"longObj\": "+EXP+".0, \"_class\": \""+LongContainer1940.class.getName()+"\"}");
+        LongContainer1940 obj = MAPPER.treeToValue(tree, LongContainer1940.class);
+        assertEquals(Long.valueOf(EXP), obj.longObj);
+    }
+}
