@@ -5,7 +5,9 @@ import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.databind.*;
 
 /**
- * Unit tests for verifying [JACKSON-132] implementation.
+ * Unit tests for verifying that basic {@link JsonUnwrapped} annotation
+ * handling works as expected; some more advanced tests are separated out
+ * to more specific test classes (like prefix/suffix handling).
  */
 public class TestUnwrapped extends BaseMapTest
 {
@@ -21,6 +23,17 @@ public class TestUnwrapped extends BaseMapTest
         }
     }
 
+    final static class Location {
+        public int x;
+        public int y;
+
+        public Location() { }
+        public Location(int x, int y) {
+            this.x = x;
+            this.y = y;
+        }
+    }
+    
     static class DeepUnwrapping
     {
         @JsonUnwrapped
@@ -43,17 +56,6 @@ public class TestUnwrapped extends BaseMapTest
             name = n;
         }
     }
-    
-    final static class Location {
-        public int x;
-        public int y;
-
-        public Location() { }
-        public Location(int x, int y) {
-            this.x = x;
-            this.y = y;
-        }
-    }
 
     // Class with two unwrapped properties
     static class TwoUnwrappedProperties {
@@ -69,6 +71,46 @@ public class TestUnwrapped extends BaseMapTest
         public String first, last;
     }
 
+    // [databind#615]
+    static class Parent {
+        @JsonUnwrapped
+        public Child c1;
+
+        public Parent() { }
+        public Parent(String str) { c1 = new Child(str); }
+    }
+
+    static class Child {
+        public String field;
+
+        public Child() { }
+        public Child(String f) { field = f; }
+    }
+
+    static class Inner {
+        public String animal;
+    }
+
+    static class Outer {
+        // @JsonProperty
+        @JsonUnwrapped
+        private Inner inner;
+    }
+
+    // [databind#1493]: case-insensitive handling
+    static class Person {
+        @JsonUnwrapped(prefix = "businessAddress.")
+        public Address businessAddress;
+    }
+
+    static class Address {
+        public String street;
+        public String addon;
+        public String zip;
+        public String town;    
+        public String country;
+    }
+
     /*
     /**********************************************************
     /* Tests, serialization
@@ -142,45 +184,35 @@ public class TestUnwrapped extends BaseMapTest
         assertEquals(2, loc.y);
     }
 
-    // 22-Apr-2013, tatu: Commented out as it can't be simply fixed; requires implementing
-    //    deep-update/merge. But leaving here to help with that effort, if/when it proceeds.
-    
-    /*
-    
-    // [Issue#211]: Actually just variant of #160
-    
-    static class Issue211Bean {
-        public String test1;
-
-        public String test2;
-        @JsonUnwrapped
-        public Issue211Unwrapped unwrapped;
-    }
-
-    static class Issue211Unwrapped {
-        public String test3;
-        public String test4;
-    }
-    
-    public void testIssue211() throws Exception
+    public void testIssue615() throws Exception
     {
-         Issue211Bean bean = new Issue211Bean();
-         bean.test1 = "Field 1";
-         bean.test2 = "Field 2";
-         Issue211Unwrapped tJackson2 = new Issue211Unwrapped();
-         tJackson2.test3 = "Field 3";
-         tJackson2.test4 = "Field 4";
-         bean.unwrapped = tJackson2;
- 
-         final String JSON = "{\"test1\": \"Field 1 merged\", \"test3\": \"Field 3 merged\"}";
-         ObjectMapper o = new ObjectMapper();
-         Issue211Bean result = o.readerForUpdating(bean).withType(Issue211Bean.class).readValue(JSON);
-         assertSame(bean, result);
-         assertEquals("Field 1 merged", result.test1);
-         assertEquals("Field 2", result.test2);
-         assertNotNull(result.unwrapped);
-         assertEquals("Field 3 merged", result.unwrapped.test3);
-         assertEquals("Field 4", result.unwrapped.test4);
-    }  
-    */
+        Parent input = new Parent("name");
+        String json = MAPPER.writeValueAsString(input);
+        Parent output = MAPPER.readValue(json, Parent.class);
+        assertEquals("name", output.c1.field);
+    }
+
+    public void testUnwrappedAsPropertyIndicator() throws Exception
+    {
+        Inner inner = new Inner();
+        inner.animal = "Zebra";
+
+        Outer outer = new Outer();
+        outer.inner = inner;
+
+        String actual = MAPPER.writeValueAsString(outer);
+
+        assertTrue(actual.contains("animal"));
+        assertTrue(actual.contains("Zebra"));
+        assertFalse(actual.contains("inner"));
+    }
+
+    // [databind#1493]: case-insensitive handling
+    public void testCaseInsensitiveUnwrap() throws Exception
+    {
+        ObjectMapper mapper = new ObjectMapper();
+        mapper.enable(MapperFeature.ACCEPT_CASE_INSENSITIVE_PROPERTIES);
+        Person p = mapper.readValue("{ }", Person.class);
+        assertNotNull(p);
+    }
 }
