@@ -5,21 +5,14 @@ import java.util.*;
 import java.util.concurrent.atomic.AtomicReference;
 
 import com.fasterxml.jackson.core.type.TypeReference;
-import com.fasterxml.jackson.databind.JavaType;
-import com.fasterxml.jackson.databind.ObjectMapper;
-import com.fasterxml.jackson.databind.type.CollectionType;
-import com.fasterxml.jackson.databind.type.HierarchicType;
-import com.fasterxml.jackson.databind.type.MapType;
-import com.fasterxml.jackson.databind.type.SimpleType;
-import com.fasterxml.jackson.databind.type.TypeFactory;
-import com.fasterxml.jackson.test.BaseTest;
+import com.fasterxml.jackson.databind.*;
 
 /**
  * Simple tests to verify that the {@link TypeFactory} constructs
  * type information as expected.
  */
 public class TestTypeFactory
-    extends BaseTest
+    extends BaseMapTest
 {    
     /*
     /**********************************************************
@@ -74,7 +67,14 @@ public class TestTypeFactory
     static class StringListBean {
         public GenericList<String> value;
     }
-    
+
+    static class CollectionLike<E> { }
+    static class MapLike<K,V> { }
+
+    static class Wrapper1297<T> {
+        public T content;
+    }
+
     /*
     /**********************************************************
     /* Unit tests
@@ -125,60 +125,21 @@ public class TestTypeFactory
         }
     }
 
-    public void testCollections()
+    // [databind#810]: Fake Map type for Properties as <String,String>
+    public void testProperties()
     {
-        // Ok, first: let's test what happens when we pass 'raw' Collection:
         TypeFactory tf = TypeFactory.defaultInstance();
-        JavaType t = tf.constructType(ArrayList.class);
-        assertEquals(CollectionType.class, t.getClass());
-        assertSame(ArrayList.class, t.getRawClass());
+        JavaType t = tf.constructType(Properties.class);
+        assertEquals(MapType.class, t.getClass());
+        assertSame(Properties.class, t.getRawClass());
 
-        // And then the proper way
-        t = tf.constructType(new TypeReference<ArrayList<String>>() { });
-        assertEquals(CollectionType.class, t.getClass());
-        assertSame(ArrayList.class, t.getRawClass());
+        MapType mt = (MapType) t;
 
-        JavaType elemType = ((CollectionType) t).getContentType();
-        assertNotNull(elemType);
-        assertSame(SimpleType.class, elemType.getClass());
-        assertSame(String.class, elemType.getRawClass());
-
-        // And alternate method too
-        t = tf.constructCollectionType(ArrayList.class, String.class);
-        assertEquals(CollectionType.class, t.getClass());
-        assertSame(String.class, ((CollectionType) t).getContentType().getRawClass());
+        // so far so good. But how about parameterization?
+        assertSame(String.class, mt.getKeyType().getRawClass());
+        assertSame(String.class, mt.getContentType().getRawClass());
     }
     
-    public void testMaps()
-    {
-        TypeFactory tf = TypeFactory.defaultInstance();
-        // Ok, first: let's test what happens when we pass 'raw' Map:
-        JavaType t = tf.constructType(HashMap.class);
-        assertEquals(MapType.class, t.getClass());
-        assertSame(HashMap.class, t.getRawClass());
-
-        // Then explicit construction
-        t = tf.constructMapType(TreeMap.class, String.class, Integer.class);
-        assertEquals(MapType.class, t.getClass());
-        assertSame(String.class, ((MapType) t).getKeyType().getRawClass());
-        assertSame(Integer.class, ((MapType) t).getContentType().getRawClass());
-
-        // And then with TypeReference
-        t = tf.constructType(new TypeReference<HashMap<String,Integer>>() { });
-        assertEquals(MapType.class, t.getClass());
-        assertSame(HashMap.class, t.getRawClass());
-        MapType mt = (MapType) t;
-        assertEquals(tf.constructType(String.class), mt.getKeyType());
-        assertEquals(tf.constructType(Integer.class), mt.getContentType());
-
-        t = tf.constructType(new TypeReference<LongValuedMap<Boolean>>() { });
-        assertEquals(MapType.class, t.getClass());
-        assertSame(LongValuedMap.class, t.getRawClass());
-        mt = (MapType) t;
-        assertEquals(tf.constructType(Boolean.class), mt.getKeyType());
-        assertEquals(tf.constructType(Long.class), mt.getContentType());
-    }
-
     public void testIterator()
     {
         TypeFactory tf = TypeFactory.defaultInstance();
@@ -194,11 +155,12 @@ public class TestTypeFactory
      * Test for verifying that parametric types can be constructed
      * programmatically
      */
+    @SuppressWarnings("deprecation")
     public void testParametricTypes()
     {
         TypeFactory tf = TypeFactory.defaultInstance();
         // first, simple class based
-        JavaType t = tf.constructParametricType(ArrayList.class, String.class); // ArrayList<String>
+        JavaType t = tf.constructParametrizedType(ArrayList.class, Collection.class, String.class); // ArrayList<String>
         assertEquals(CollectionType.class, t.getClass());
         JavaType strC = tf.constructType(String.class);
         assertEquals(1, t.containedTypeCount());
@@ -206,7 +168,7 @@ public class TestTypeFactory
         assertNull(t.containedType(1));
 
         // Then using JavaType
-        JavaType t2 = tf.constructParametricType(Map.class, strC, t); // Map<String,ArrayList<String>>
+        JavaType t2 = tf.constructParametrizedType(Map.class, Map.class, strC, t); // Map<String,ArrayList<String>>
         // should actually produce a MapType
         assertEquals(MapType.class, t2.getClass());
         assertEquals(2, t2.containedTypeCount());
@@ -215,27 +177,29 @@ public class TestTypeFactory
         assertNull(t2.containedType(2));
 
         // and then custom generic type as well
-        JavaType custom = tf.constructParametricType(SingleArgGeneric.class, String.class);
+        JavaType custom = tf.constructParametrizedType(SingleArgGeneric.class, SingleArgGeneric.class,
+                String.class);
         assertEquals(SimpleType.class, custom.getClass());
         assertEquals(1, custom.containedTypeCount());
         assertEquals(strC, custom.containedType(0));
         assertNull(custom.containedType(1));
+
         // should also be able to access variable name:
         assertEquals("X", custom.containedTypeName(0));
 
         // And finally, ensure that we can't create invalid combinations
         try {
             // Maps must take 2 type parameters, not just one
-            tf.constructParametricType(Map.class, strC);
+            tf.constructParametrizedType(Map.class, Map.class, strC);
         } catch (IllegalArgumentException e) {
-            verifyException(e, "Need exactly 2 parameter types for Map types");
+            verifyException(e, "Cannot create TypeBindings for class java.util.Map");
         }
 
         try {
             // Type only accepts one type param
-            tf.constructParametricType(SingleArgGeneric.class, strC, strC);
+            tf.constructParametrizedType(SingleArgGeneric.class, SingleArgGeneric.class, strC, strC);
         } catch (IllegalArgumentException e) {
-            verifyException(e, "expected 1 parameters, was given 2");
+            verifyException(e, "Cannot create TypeBindings for class ");
         }
     }
 
@@ -267,90 +231,181 @@ public class TestTypeFactory
         assertEquals("java.util.EnumMap<com.fasterxml.jackson.databind.type.TestTypeFactory$EnumForCanonical,java.lang.String>",
                 can);
         assertEquals(t, tf.constructFromCanonical(can));
-        
+
+        // [databind#1941]: allow "raw" types too
+
+        t = tf.constructFromCanonical("java.util.List");
+        assertEquals(List.class, t.getRawClass());
+        assertEquals(CollectionType.class, t.getClass());
+        // 01-Mar-2018, tatu: not 100% should we expect type parameters here...
+        //    But currently we do NOT get any
+        /*
+        assertEquals(1, t.containedTypeCount());
+        assertEquals(Object.class, t.containedType(0).getRawClass());
+        */
+        assertEquals(Object.class, t.getContentType().getRawClass());
+        can = t.toCanonical();
+        assertEquals("java.util.List<java.lang.Object>", can);
+        assertEquals(t, tf.constructFromCanonical(can));
+    }
+
+    // [databind#1768]
+    @SuppressWarnings("serial")
+    public void testCanonicalWithSpaces()
+    {
+        TypeFactory tf = TypeFactory.defaultInstance();
+        Object objects = new TreeMap<Object, Object>() { }; // to get subtype
+        String reflectTypeName = objects.getClass().getGenericSuperclass().toString();
+        JavaType t1 = tf.constructType(objects.getClass().getGenericSuperclass());
+        // This will throw an Exception if you don't remove all white spaces from the String.
+        JavaType t2 = tf.constructFromCanonical(reflectTypeName);
+        assertNotNull(t2);
+        assertEquals(t2, t1);
     }
 
     /*
     /**********************************************************
-    /* Unit tests: low-level inheritance resolution
+    /* Unit tests: collection type parameter resolution
     /**********************************************************
      */
 
-    public void testSuperTypeDetectionClass()
+    public void testCollections()
     {
+        // Ok, first: let's test what happens when we pass 'raw' Collection:
         TypeFactory tf = TypeFactory.defaultInstance();
-        HierarchicType sub = tf._findSuperTypeChain(MyStringIntMap.class, HashMap.class);
-        assertNotNull(sub);
-        assertEquals(2, _countSupers(sub));
-        assertSame(MyStringIntMap.class, sub.getRawClass());
-        HierarchicType sup = sub.getSuperType();
-        assertSame(MyStringXMap.class, sup.getRawClass());
-        HierarchicType sup2 = sup.getSuperType();
-        assertSame(HashMap.class, sup2.getRawClass());
-        assertNull(sup2.getSuperType());
-    }
+        JavaType t = tf.constructType(ArrayList.class);
+        assertEquals(CollectionType.class, t.getClass());
+        assertSame(ArrayList.class, t.getRawClass());
 
-    public void testSuperTypeDetectionInterface()
-    {
-        // List first
-        TypeFactory tf = TypeFactory.defaultInstance();
-        HierarchicType sub = tf._findSuperTypeChain(MyList.class, List.class);
-        assertNotNull(sub);
-        assertEquals(2, _countSupers(sub));
-        assertSame(MyList.class, sub.getRawClass());
-        HierarchicType sup = sub.getSuperType();
-        assertSame(IntermediateList.class, sup.getRawClass());
-        HierarchicType sup2 = sup.getSuperType();
-        assertSame(List.class, sup2.getRawClass());
-        assertNull(sup2.getSuperType());
-        
-        // Then Map
-        sub = tf._findSuperTypeChain(MyMap.class, Map.class);
-        assertNotNull(sub);
-        assertEquals(2, _countSupers(sub));
-        assertSame(MyMap.class, sub.getRawClass());
-        sup = sub.getSuperType();
-        assertSame(IntermediateMap.class, sup.getRawClass());
-        sup2 = sup.getSuperType();
-        assertSame(Map.class, sup2.getRawClass());
-        assertNull(sup2.getSuperType());
-    }
+        // And then the proper way
+        t = tf.constructType(new TypeReference<ArrayList<String>>() { });
+        assertEquals(CollectionType.class, t.getClass());
+        assertSame(ArrayList.class, t.getRawClass());
 
-    public void testAtomicArrayRefParameterDetection()
-    {
-        TypeFactory tf = TypeFactory.defaultInstance();
-        JavaType type = tf.constructType(new TypeReference<AtomicReference<long[]>>() { });
-        HierarchicType sub = tf._findSuperTypeChain(type.getRawClass(), AtomicReference.class);
-        assertNotNull(sub);
-        assertEquals(0, _countSupers(sub));
-        assertTrue(AtomicReference.class.isAssignableFrom(type.getRawClass()));
-        assertNull(sub.getSuperType());
-    }
+        JavaType elemType = ((CollectionType) t).getContentType();
+        assertNotNull(elemType);
+        assertSame(SimpleType.class, elemType.getClass());
+        assertSame(String.class, elemType.getRawClass());
 
-    private int _countSupers(HierarchicType t)
-    {
-        int depth = 0;
-        for (HierarchicType sup = t.getSuperType(); sup != null; sup = sup.getSuperType()) {
-            ++depth;
-        }
-        return depth;
+        // And alternate method too
+        t = tf.constructCollectionType(ArrayList.class, String.class);
+        assertEquals(CollectionType.class, t.getClass());
+        assertSame(String.class, ((CollectionType) t).getContentType().getRawClass());
     }
     
+    // since 2.7
+    public void testCollectionTypesRefined()
+    {
+        TypeFactory tf = newTypeFactory();
+        JavaType type = tf.constructType(new TypeReference<List<Long>>() { });
+        assertEquals(List.class, type.getRawClass());
+        assertEquals(Long.class, type.getContentType().getRawClass());
+        // No super-class, since it's an interface:
+        assertNull(type.getSuperClass());
+
+        // But then refine to reflect sub-classing
+        JavaType subtype = tf.constructSpecializedType(type, ArrayList.class);
+        assertEquals(ArrayList.class, subtype.getRawClass());
+        assertEquals(Long.class, subtype.getContentType().getRawClass());
+
+        // but with refinement, should have non-null super class
+        JavaType superType = subtype.getSuperClass();
+        assertNotNull(superType);
+        assertEquals(AbstractList.class, superType.getRawClass());
+    }
+
     /*
     /**********************************************************
-    /* Unit tests: map/collection type parameter resolution
+    /* Unit tests: map type parameter resolution
     /**********************************************************
      */
-
-    public void testMapTypesSimple()
+    
+    public void testMaps()
     {
-        TypeFactory tf = TypeFactory.defaultInstance();
+        TypeFactory tf = newTypeFactory();
+
+        // Ok, first: let's test what happens when we pass 'raw' Map:
+        JavaType t = tf.constructType(HashMap.class);
+        assertEquals(MapType.class, t.getClass());
+        assertSame(HashMap.class, t.getRawClass());
+
+        // Then explicit construction
+        t = tf.constructMapType(TreeMap.class, String.class, Integer.class);
+        assertEquals(MapType.class, t.getClass());
+        assertSame(String.class, ((MapType) t).getKeyType().getRawClass());
+        assertSame(Integer.class, ((MapType) t).getContentType().getRawClass());
+
+        // And then with TypeReference
+        t = tf.constructType(new TypeReference<HashMap<String,Integer>>() { });
+        assertEquals(MapType.class, t.getClass());
+        assertSame(HashMap.class, t.getRawClass());
+        MapType mt = (MapType) t;
+        assertEquals(tf.constructType(String.class), mt.getKeyType());
+        assertEquals(tf.constructType(Integer.class), mt.getContentType());
+
+        t = tf.constructType(new TypeReference<LongValuedMap<Boolean>>() { });
+        assertEquals(MapType.class, t.getClass());
+        assertSame(LongValuedMap.class, t.getRawClass());
+        mt = (MapType) t;
+        assertEquals(tf.constructType(Boolean.class), mt.getKeyType());
+        assertEquals(tf.constructType(Long.class), mt.getContentType());
+
         JavaType type = tf.constructType(new TypeReference<Map<String,Boolean>>() { });
         MapType mapType = (MapType) type;
         assertEquals(tf.constructType(String.class), mapType.getKeyType());
         assertEquals(tf.constructType(Boolean.class), mapType.getContentType());
     }
 
+    // since 2.7
+    public void testMapTypesRefined()
+    {
+        TypeFactory tf = newTypeFactory();
+        JavaType type = tf.constructType(new TypeReference<Map<String,List<Integer>>>() { });
+        MapType mapType = (MapType) type;
+        assertEquals(Map.class, mapType.getRawClass());
+        assertEquals(String.class, mapType.getKeyType().getRawClass());
+        assertEquals(List.class, mapType.getContentType().getRawClass());
+        assertEquals(Integer.class, mapType.getContentType().getContentType().getRawClass());
+        // No super-class, since it's an interface:
+        assertNull(type.getSuperClass());
+        
+        // But then refine to reflect sub-classing
+        JavaType subtype = tf.constructSpecializedType(type, LinkedHashMap.class);
+        assertEquals(LinkedHashMap.class, subtype.getRawClass());
+        assertEquals(String.class, subtype.getKeyType().getRawClass());
+        assertEquals(List.class, subtype.getContentType().getRawClass());
+        assertEquals(Integer.class, subtype.getContentType().getContentType().getRawClass());
+
+        // but with refinement, should have non-null super class
+
+        JavaType superType = subtype.getSuperClass();
+        assertNotNull(superType);
+        assertEquals(HashMap.class, superType.getRawClass());
+        // which also should have proper typing
+        assertEquals(String.class, superType.getKeyType().getRawClass());
+        assertEquals(List.class, superType.getContentType().getRawClass());
+        assertEquals(Integer.class, superType.getContentType().getContentType().getRawClass());
+    }
+
+    public void testTypeGeneralization()
+    {
+        TypeFactory tf = newTypeFactory();
+        MapType t = tf.constructMapType(HashMap.class, String.class, Long.class);
+        JavaType superT = tf.constructGeneralizedType(t, Map.class);
+        assertEquals(String.class, superT.getKeyType().getRawClass());
+        assertEquals(Long.class, superT.getContentType().getRawClass());
+
+        assertSame(t, tf.constructGeneralizedType(t, HashMap.class));
+
+        // plus check there is super/sub relationship
+        try {
+            tf.constructGeneralizedType(t, TreeMap.class);
+            fail("Should not pass");
+        } catch (IllegalArgumentException e) {
+            verifyException(e, "not a super-type of");
+        }
+    }
+
     public void testMapTypesRaw()
     {
         TypeFactory tf = TypeFactory.defaultInstance();
@@ -367,9 +422,10 @@ public class TestTypeFactory
         MapType mapType = (MapType) type;
         assertEquals(tf.constructType(String.class), mapType.getKeyType());
         assertEquals(tf.constructType(Long.class), mapType.getContentType());
-
+        
         type = tf.constructType(MapInterface.class);
         mapType = (MapType) type;
+
         assertEquals(tf.constructType(String.class), mapType.getKeyType());
         assertEquals(tf.constructType(Integer.class), mapType.getContentType());
 
@@ -431,7 +487,20 @@ public class TestTypeFactory
         assertEquals(1, list.size());
         assertEquals("...", list.get(0));
     }
-    
+
+    public void testSneakySelfRefs() throws Exception
+    {
+        ObjectMapper mapper = new ObjectMapper();
+        String json = mapper.writeValueAsString(new SneakyBean2());
+        assertEquals("{\"foobar\":null}", json);
+    }
+
+    /*
+    /**********************************************************
+    /* Unit tests: handling of specific JDK types
+    /**********************************************************
+     */
+
     public void testAtomicArrayRefParameters()
     {
         TypeFactory tf = TypeFactory.defaultInstance();
@@ -442,11 +511,18 @@ public class TestTypeFactory
         assertEquals(tf.constructType(long[].class), params[0]);
     }
 
-    public void testSneakySelfRefs() throws Exception
+    static abstract class StringIntMapEntry implements Map.Entry<String,Integer> { }
+    
+    public void testMapEntryResolution()
     {
-        ObjectMapper mapper = new ObjectMapper();
-        String json = mapper.writeValueAsString(new SneakyBean2());
-        assertEquals("{\"foobar\":null}", json);
+        TypeFactory tf = TypeFactory.defaultInstance();
+        JavaType t = tf.constructType(StringIntMapEntry.class);
+        JavaType mapEntryType = t.findSuperType(Map.Entry.class);
+        assertNotNull(mapEntryType);
+        assertTrue(mapEntryType.hasGenericTypes());
+        assertEquals(2, mapEntryType.containedTypeCount());
+        assertEquals(String.class, mapEntryType.containedType(0).getRawClass());
+        assertEquals(Integer.class, mapEntryType.containedType(1).getRawClass());
     }
 
     /*
@@ -461,8 +537,12 @@ public class TestTypeFactory
         JavaType type = tf.constructRawCollectionType(ArrayList.class);
         assertTrue(type.isContainerType());
         assertEquals(TypeFactory.unknownType(), type.getContentType());
+        type = tf.constructRawCollectionLikeType(CollectionLike.class); // must have type vars
+        assertTrue(type.isCollectionLikeType());
+        assertEquals(TypeFactory.unknownType(), type.getContentType());
 
-        type = tf.constructRawCollectionLikeType(String.class); // class doesn't really matter
+        // actually, should also allow "no type vars" case
+        type = tf.constructRawCollectionLikeType(String.class);
         assertTrue(type.isCollectionLikeType());
         assertEquals(TypeFactory.unknownType(), type.getContentType());
     }
@@ -475,7 +555,13 @@ public class TestTypeFactory
         assertEquals(TypeFactory.unknownType(), type.getKeyType());
         assertEquals(TypeFactory.unknownType(), type.getContentType());
 
-        type = tf.constructRawMapLikeType(String.class); // class doesn't really matter
+        type = tf.constructRawMapLikeType(MapLike.class); // must have type vars
+        assertTrue(type.isMapLikeType());
+        assertEquals(TypeFactory.unknownType(), type.getKeyType());
+        assertEquals(TypeFactory.unknownType(), type.getContentType());
+
+        // actually, should also allow "no type vars" case
+        type = tf.constructRawMapLikeType(String.class);
         assertTrue(type.isMapLikeType());
         assertEquals(TypeFactory.unknownType(), type.getKeyType());
         assertEquals(TypeFactory.unknownType(), type.getContentType());
@@ -508,15 +594,25 @@ public class TestTypeFactory
         assertSame(t2, tf.moreSpecificType(t2, t1));
     }
 
-    // [Issue#489]
+    // [databind#489]
     public void testCacheClearing()
     {
         TypeFactory tf = TypeFactory.defaultInstance().withModifier(null);
         assertEquals(0, tf._typeCache.size());
         tf.constructType(getClass());
-        assertEquals(1, tf._typeCache.size());
+        // 19-Oct-2015, tatu: This is pretty fragile but
+        assertEquals(6, tf._typeCache.size());
         tf.clearCache();
         assertEquals(0, tf._typeCache.size());
     }
+
+    // for [databind#1297]
+    public void testRawMapType()
+    {
+        TypeFactory tf = TypeFactory.defaultInstance().withModifier(null); // to get a new copy
+
+        JavaType type = tf.constructParametricType(Wrapper1297.class, Map.class);
+        assertNotNull(type);
+        assertEquals(Wrapper1297.class, type.getRawClass());
+    }
 }
-        
\ No newline at end of file
