@@ -5,11 +5,8 @@ import java.util.*;
 
 
 import com.fasterxml.jackson.databind.BaseMapTest;
-import com.fasterxml.jackson.databind.util.ISO8601DateFormat;
 
-/**
- * @see ISO8601DateFormat
- */
+@SuppressWarnings("deprecation")
 public class ISO8601DateFormatTest extends BaseMapTest
 {
     private ISO8601DateFormat df;
@@ -33,6 +30,16 @@ public class ISO8601DateFormatTest extends BaseMapTest
     public void testParse() throws Exception {
         Date result = df.parse("2007-08-13T19:51:23Z");
         assertEquals(date, result);
+
+        // Test parsing date-only values with and without a timezone designation
+        Date dateOnly = df.parse("2007-08-14");
+        Calendar cal = new GregorianCalendar(2007, 8-1, 14);
+        assertEquals(cal.getTime(), dateOnly);
+
+        dateOnly = df.parse("2007-08-14Z");
+        cal = new GregorianCalendar(2007, 8-1, 14);
+        cal.setTimeZone(TimeZone.getTimeZone("GMT"));
+        assertEquals(cal.getTime(), dateOnly);
     }
 
     public void testPartialParse() throws Exception {
@@ -49,4 +56,10 @@ public class ISO8601DateFormatTest extends BaseMapTest
         assertSame(df, clone);
     }
 
+    public void testHashCodeEquals() throws Exception {
+        // for [databind#1130]
+        DateFormat defaultDF = StdDateFormat.instance;
+        defaultDF.hashCode();
+        assertTrue(defaultDF.equals(defaultDF));
+    }
 }
